@@ -1,9 +1,9 @@
 package Slic3r::GUI::3DScene::Base;
 use strict;
 use warnings;
-
 use Wx::Event qw(EVT_PAINT EVT_SIZE EVT_ERASE_BACKGROUND EVT_IDLE EVT_MOUSEWHEEL EVT_MOUSE_EVENTS);
 # must load OpenGL *before* Wx::GLCanvas
+
 use OpenGL qw(:glconstants :glfunctions :glufunctions :gluconstants);
 use base qw(Wx::GLCanvas Class::Accessor);
 use Math::Trig qw(asin);
@@ -11,9 +11,8 @@ use List::Util qw(reduce min max first);
 use Slic3r::Geometry qw(X Y Z MIN MAX triangle_normal normalize deg2rad tan scale unscale scaled_epsilon);
 use Slic3r::Geometry::Clipper qw(offset_ex intersection_pl);
 use Wx::GLCanvas qw(:all);
- 
+
 __PACKAGE__->mk_accessors( qw(_quat _dirty init
-                              enable_cutting
                               enable_picking
                               enable_moving
                               on_viewport_changed
@@ -24,6 +23,7 @@ __PACKAGE__->mk_accessors( qw(_quat _dirty init
                               on_move
                               volumes
                               _sphi _stheta
+                              cutting_plane_axis
                               cutting_plane_z
                               cut_lines_vertices
                               bed_shape
@@ -44,9 +44,25 @@ __PACKAGE__->mk_accessors( qw(_quat _dirty init
 use constant TRACKBALLSIZE  => 0.8;
 use constant TURNTABLE_MODE => 1;
 use constant GROUND_Z       => -0.02;
-use constant DEFAULT_COLOR  => [1,1,0];
-use constant SELECTED_COLOR => [0,1,0,1];
-use constant HOVER_COLOR    => [0.4,0.9,0,1];
+use constant PI             => 3.1415927;
+
+# Constant to determine if Vertex Buffer objects are used to draw
+# bed grid and the cut plane for object separation.
+use constant HAS_VBO        => eval { glGenBuffersARB_p(0); GL_ARRAY_BUFFER_ARB; 1 };
+
+# phi / theta angles to orient the camera.
+use constant VIEW_TOP        => [0.0,0.0];
+use constant VIEW_BOTTOM     => [0.0,180.0];
+use constant VIEW_LEFT       => [90.0,90.0];
+use constant VIEW_RIGHT      => [-90.0,90.0];
+use constant VIEW_FRONT      => [0.0,90.0];
+use constant VIEW_BACK       => [180.0,90.0];
+use constant VIEW_DIAGONAL   => [45.0,45.0];
+
+# Color Scheme
+use Slic3r::GUI::ColorScheme;
+
+use constant GIMBAL_LOCK_THETA_MAX => 170;
 
 # make OpenGL::Array thread-safe
 {
@@ -56,12 +72,41 @@ use constant HOVER_COLOR    => [0.4,0.9,0,1];
 
 sub new {
     my ($class, $parent) = @_;
+
+    if ( ( defined $Slic3r::GUI::Settings->{_}{colorscheme} ) && ( Slic3r::GUI::ColorScheme->can($Slic3r::GUI::Settings->{_}{colorscheme}) ) ) {
+        my $myGetSchemeName = \&{"Slic3r::GUI::ColorScheme::$Slic3r::GUI::Settings->{_}{colorscheme}"};
+        $myGetSchemeName->();
+    } else {
+        Slic3r::GUI::ColorScheme->getDefault();
+    }
     
+    # We can only enable multi sample anti aliasing with wxWidgets 3.0.3 and with a hacked Wx::GLCanvas,
+    # which exports some new WX_GL_XXX constants, namely WX_GL_SAMPLE_BUFFERS and WX_GL_SAMPLES.
+    my $can_multisample =
+        Wx::wxVERSION >= 3.000003 &&
+        defined Wx::GLCanvas->can('WX_GL_SAMPLE_BUFFERS') &&
+        defined Wx::GLCanvas->can('WX_GL_SAMPLES');
+    my $attrib = [WX_GL_RGBA, WX_GL_DOUBLEBUFFER, WX_GL_DEPTH_SIZE, 24];
+    if ($can_multisample) {
+        # Request a window with multi sampled anti aliasing. This is a new feature in Wx 3.0.3 (backported from 3.1.0).
+        # Use eval to avoid compilation, if the subs WX_GL_SAMPLE_BUFFERS and WX_GL_SAMPLES are missing.
+        eval 'push(@$attrib, (WX_GL_SAMPLE_BUFFERS, 1, WX_GL_SAMPLES, 4));';
+    }
+    # wxWidgets expect the attrib list to be ended by zero.
+    push(@$attrib, 0);
+
     # we request a depth buffer explicitely because it looks like it's not created by 
     # default on Linux, causing transparency issues
-    my $self = $class->SUPER::new($parent, -1, Wx::wxDefaultPosition, Wx::wxDefaultSize, 0, "",
-        [WX_GL_RGBA, WX_GL_DOUBLEBUFFER, WX_GL_DEPTH_SIZE, 16, 0]);
-   
+    my $self = $class->SUPER::new($parent, -1, Wx::wxDefaultPosition, Wx::wxDefaultSize, 0, "", $attrib);
+
+    if (Wx::wxVERSION >= 3.000003) {
+        # Wx 3.0.3 contains an ugly hack to support some advanced OpenGL attributes through the attribute list.
+        # The attribute list is transferred between the wxGLCanvas and wxGLContext constructors using a single static array s_wglContextAttribs.
+        # Immediatelly force creation of the OpenGL context to consume the static variable s_wglContextAttribs.
+        $self->GetContext();
+    }
+
+    $self->{can_multisample} = $can_multisample;
     $self->background(1);
     $self->_quat((0, 0, 0, 1));
     $self->_stheta(45);
@@ -89,6 +134,9 @@ sub new {
         
         # Calculate the zoom delta and apply it to the current zoom factor
         my $zoom = $e->GetWheelRotation() / $e->GetWheelDelta();
+        if ($Slic3r::GUI::Settings->{_}{invert_zoom}) {
+            $zoom *= -1;
+        }
         $zoom = max(min($zoom, 4), -4);
         $zoom /= 10;
         $self->_zoom($self->_zoom / (1-$zoom));
@@ -114,6 +162,7 @@ sub new {
         $self->Refresh;
     });
     EVT_MOUSE_EVENTS($self, \&mouse_event);
+
     
     return $self;
 }
@@ -128,7 +177,15 @@ sub mouse_event {
     } elsif ($e->LeftDClick) {
         $self->on_double_click->()
             if $self->on_double_click;
-    } elsif ($e->LeftDown || $e->RightDown) {
+    } elsif ($e->MiddleDClick) {
+        if (@{$self->volumes}) {
+            $self->zoom_to_volumes;
+        } else {
+            $self->zoom_to_bed;
+        }
+        $self->_dirty(1);
+        $self->Refresh;
+    } elsif (($e->LeftDown || $e->RightDown) && not $e->ShiftDown) {
         # If user pressed left or right button we first check whether this happened
         # on a volume or not.
         my $volume_idx = $self->_hover_volume_idx // -1;
@@ -190,14 +247,23 @@ sub mouse_event {
         $self->_dragged(1);
         $self->Refresh;
     } elsif ($e->Dragging) {
-        if ($e->LeftIsDown) {
+        if ($e->AltDown) {
+            # Move the camera center on the Z axis based on mouse Y axis movement
+            if (defined $self->_drag_start_pos) {
+                my $orig = $self->_drag_start_pos;
+                $self->_camera_target->translate(0, 0, $pos->y - $orig->y);
+                $self->on_viewport_changed->() if $self->on_viewport_changed;
+                $self->Refresh;
+            }
+            $self->_drag_start_pos($pos);
+        } elsif ($e->LeftIsDown) {
             # if dragging over blank area with left button, rotate
             if (defined $self->_drag_start_pos) {
                 my $orig = $self->_drag_start_pos;
                 if (TURNTABLE_MODE) {
                     $self->_sphi($self->_sphi + ($pos->x - $orig->x) * TRACKBALLSIZE);
                     $self->_stheta($self->_stheta - ($pos->y - $orig->y) * TRACKBALLSIZE);        #-
-                    $self->_stheta(150) if $self->_stheta > 150;
+                    $self->_stheta(GIMBAL_LOCK_THETA_MAX) if $self->_stheta > GIMBAL_LOCK_THETA_MAX;
                     $self->_stheta(0) if $self->_stheta < 0;
                 } else {
                     my $size = $self->GetClientSize;
@@ -247,7 +313,7 @@ sub mouse_event {
         $self->_dragged(undef);
     } elsif ($e->Moving) {
         $self->_mouse_pos($pos);
-        $self->Refresh;
+        $self->Refresh if $self->enable_picking;
     } else {
         $e->Skip();
     }
@@ -271,19 +337,76 @@ sub set_viewport_from_scene {
     $self->_dirty(1);
 }
 
+sub zoom{
+    my ($self, $direction) = @_;
+    if( $direction eq 'in'){
+        $self->_zoom($self->_zoom / (1-0.3));
+    }
+    elsif($direction eq 'out'){
+        $self->_zoom($self->_zoom / (1+0.3));
+    }
+    $self->on_viewport_changed->() if $self->on_viewport_changed;
+    $self->_dirty(1);
+    $self->Refresh;
+}
+
+# Set the camera to a default orientation,
+# zoom to volumes.
+sub select_view {
+    my ($self, $direction) = @_;
+    my $dirvec;
+    if (ref($direction)) {
+        $dirvec = $direction;
+    } else {
+        if ($direction eq 'top') {
+            $dirvec = VIEW_TOP;
+        } elsif ($direction eq 'bottom') {
+            $dirvec = VIEW_BOTTOM;
+        } elsif ($direction eq 'left') {
+            $dirvec = VIEW_LEFT;
+        } elsif ($direction eq 'right') {
+            $dirvec = VIEW_RIGHT;
+        } elsif ($direction eq 'front') {
+            $dirvec = VIEW_FRONT;
+        } elsif ($direction eq 'back') {
+            $dirvec = VIEW_BACK;
+        } elsif ($direction eq 'diagonal') {
+            $dirvec = VIEW_DIAGONAL;
+        }
+    }
+    $self->_sphi($dirvec->[0]);
+    $self->_stheta($dirvec->[1]);
+    
+    # Avoid gimbal lock.
+    $self->_stheta(GIMBAL_LOCK_THETA_MAX) if $self->_stheta > GIMBAL_LOCK_THETA_MAX;
+    $self->_stheta(0) if $self->_stheta < 0;
+    
+    # View everything.
+    $self->volumes_bounding_box->defined
+        ? $self->zoom_to_volumes
+        : $self->zoom_to_bed;
+    
+    $self->on_viewport_changed->() if $self->on_viewport_changed;
+    $self->_dirty(1);
+    $self->Refresh;
+}
+
 sub zoom_to_bounding_box {
     my ($self, $bb) = @_;
     
     # calculate the zoom factor needed to adjust viewport to
     # bounding box
-    my $max_size = max(@{$bb->size}) * 2;
+    my $max_size = max(@{$bb->size}) * 1.05;
     my $min_viewport_size = min($self->GetSizeWH);
-    $self->_zoom($min_viewport_size / $max_size);
+    if ($max_size != 0) {
+        # only re-zoom if we have a valid bounding box, avoid a divide by 0 error.
+        $self->_zoom($min_viewport_size / $max_size);
     
-    # center view around bounding box center
-    $self->_camera_target($bb->center);
+        # center view around bounding box center
+        $self->_camera_target($bb->center);
     
-    $self->on_viewport_changed->() if $self->on_viewport_changed;
+        $self->on_viewport_changed->() if $self->on_viewport_changed;
+    }
 }
 
 sub zoom_to_bed {
@@ -319,7 +442,9 @@ sub bed_bounding_box {
     my ($self) = @_;
     
     my $bb = Slic3r::Geometry::BoundingBoxf3->new;
-    $bb->merge_point(Slic3r::Pointf3->new(@$_, 0)) for @{$self->bed_shape};
+    if ($self->bed_shape) {
+        $bb->merge_point(Slic3r::Pointf3->new(@$_, 0)) for @{$self->bed_shape};
+    }
     return $bb;
 }
 
@@ -402,25 +527,35 @@ sub select_volume {
 }
 
 sub SetCuttingPlane {
-    my ($self, $z) = @_;
+    my ($self, $axis, $z, $expolygons) = @_;
     
+    $self->cutting_plane_axis($axis);
     $self->cutting_plane_z($z);
     
-    # perform cut and cache section lines
+    # grow slices in order to display them better
+    $expolygons = offset_ex([ map @$_, @$expolygons ], scale 0.1);
+    
+    my $bb = $self->volumes_bounding_box;
+    
     my @verts = ();
-    foreach my $volume (@{$self->volumes}) {
-        foreach my $volume (@{$self->volumes}) {
-            next if !$volume->mesh;
-            my $expolygons = $volume->mesh->slice([ $z - $volume->origin->z ])->[0];
-            $expolygons = offset_ex([ map @$_, @$expolygons ], scale 0.1);
-            
-            foreach my $line (map @{$_->lines}, map @$_, @$expolygons) {
-                push @verts, (
-                    unscale($line->a->x), unscale($line->a->y), $z,  #))
-                    unscale($line->b->x), unscale($line->b->y), $z,  #))
-                );
-            }
+    foreach my $line (map @{$_->lines}, map @$_, @$expolygons) {
+        if ($axis == X) {
+            push @verts, (
+                $bb->x_min + $z, unscale($line->a->x), unscale($line->a->y),  #))
+                $bb->x_min + $z, unscale($line->b->x), unscale($line->b->y),  #))
+            );
+        } elsif ($axis == Y) {
+            push @verts, (
+                unscale($line->a->y), $bb->y_min + $z, unscale($line->a->x),  #))
+                unscale($line->b->y), $bb->y_min + $z, unscale($line->b->x),  #))
+            );
+        } else {
+            push @verts, (
+                unscale($line->a->x), unscale($line->a->y), $z,  #))
+                unscale($line->b->x), unscale($line->b->y), $z,  #))
+            );
         }
+        
     }
     $self->cut_lines_vertices(OpenGL::Array->new_list(GL_FLOAT, @verts));
 }
@@ -616,10 +751,10 @@ sub InitGL {
     # Set antialiasing/multisampling
     glDisable(GL_LINE_SMOOTH);
     glDisable(GL_POLYGON_SMOOTH);
-    glEnable(GL_MULTISAMPLE);
+    glEnable(GL_MULTISAMPLE) if ($self->{can_multisample});
     
     # ambient lighting
-    glLightModelfv_p(GL_LIGHT_MODEL_AMBIENT, 0.3, 0.3, 0.3, 1);
+    glLightModelfv_p(GL_LIGHT_MODEL_AMBIENT, 0.1, 0.1, 0.1, 1);
     
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
@@ -627,21 +762,21 @@ sub InitGL {
     
     # light from camera
     glLightfv_p(GL_LIGHT1, GL_POSITION, 1, 0, 1, 0);
-    glLightfv_p(GL_LIGHT1, GL_SPECULAR, 0.3, 0.3, 0.3, 1);
-    glLightfv_p(GL_LIGHT1, GL_DIFFUSE,  0.2, 0.2, 0.2, 1);
+    glLightfv_p(GL_LIGHT1, GL_SPECULAR, 0.8, 0.8, 0.8, 1);
+    glLightfv_p(GL_LIGHT1, GL_DIFFUSE,  0.4, 0.4, 0.4, 1);
     
-    # Enables Smooth Color Shading; try GL_FLAT for (lack of) fun.
+    # Enables Smooth Color Shading; try GL_FLAT for (lack of) fun. Default: GL_SMOOTH
     glShadeModel(GL_SMOOTH);
     
-    glMaterialfv_p(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, 0.5, 0.3, 0.3, 1);
+    glMaterialfv_p(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, 0.3, 0.3, 0.3, 1);
     glMaterialfv_p(GL_FRONT_AND_BACK, GL_SPECULAR, 1, 1, 1, 1);
     glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50);
-    glMaterialfv_p(GL_FRONT_AND_BACK, GL_EMISSION, 0.1, 0, 0, 0.9);
+    glMaterialfv_p(GL_FRONT_AND_BACK, GL_EMISSION, 0.1, 0.1, 0.1, 0.9);
     
     # A handy trick -- have surface material mirror the color.
     glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
     glEnable(GL_COLOR_MATERIAL);
-    glEnable(GL_MULTISAMPLE);
+    glEnable(GL_MULTISAMPLE) if ($self->{can_multisample});
 }
  
 sub Render {
@@ -653,7 +788,12 @@ sub Render {
     $self->SetCurrent($context);
     $self->InitGL;
     
-    glClearColor(1, 1, 1, 1);
+    if($SOLID_BACKGROUNDCOLOR == 1){
+        glClearColor(@BACKGROUND_COLOR, 0);
+    } else {
+        glClearColor(1, 1, 1, 1);
+    }
+    
     glClearDepth(1);
     glDepthFunc(GL_LESS);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -676,6 +816,7 @@ sub Render {
     glLightfv_p(GL_LIGHT0, GL_DIFFUSE,  0.5, 0.5, 0.5, 1);
     
     if ($self->enable_picking) {
+        glDisable(GL_MULTISAMPLE) if ($self->{can_multisample});
         glDisable(GL_LIGHTING);
         $self->draw_volumes(1);
         glFlush();
@@ -705,7 +846,7 @@ sub Render {
     }
     
     # draw fixed background
-    if ($self->background) {
+    if ($SOLID_BACKGROUNDCOLOR == 0 && $self->background) {
         glDisable(GL_LIGHTING);
         glPushMatrix();
         glLoadIdentity();
@@ -715,10 +856,10 @@ sub Render {
         glLoadIdentity();
         
         glBegin(GL_QUADS);
-        glColor3f(0.0,0.0,0.0);
+        glColor3f( @BOTTOM_COLOR ); # bottom color
         glVertex2f(-1.0,-1.0);
         glVertex2f(1,-1.0);
-        glColor3f(10/255,98/255,144/255);
+        glColor3f( @TOP_COLOR ); # top color
         glVertex2f(1, 1);
         glVertex2f(-1.0, 1);
         glEnd();
@@ -741,9 +882,19 @@ sub Render {
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         
         glEnableClientState(GL_VERTEX_ARRAY);
-        glColor4f(0.8, 0.6, 0.5, 0.4);
+        my $triangle_vertex;
+        if (HAS_VBO) {
+            ($triangle_vertex) =
+                glGenBuffersARB_p(1);
+            $self->bed_triangles->bind($triangle_vertex);
+            glBufferDataARB_p(GL_ARRAY_BUFFER_ARB, $self->bed_triangles, GL_STATIC_DRAW_ARB);
+            glVertexPointer_c(3, GL_FLOAT, 0, 0);
+        } else {
+            # fall back on old behavior
+            glVertexPointer_c(3, GL_FLOAT, 0, $self->bed_triangles->ptr());
+        }
+        glColor4f( @GROUND_COLOR );
         glNormal3d(0,0,1);
-        glVertexPointer_p(3, $self->bed_triangles);
         glDrawArrays(GL_TRIANGLES, 0, $self->bed_triangles->elements / 3);
         glDisableClientState(GL_VERTEX_ARRAY);
         
@@ -752,14 +903,32 @@ sub Render {
         glEnable(GL_DEPTH_TEST);
     
         # draw grid
-        glLineWidth(3);
-        glColor4f(0.2, 0.2, 0.2, 0.4);
+        glLineWidth(2);
         glEnableClientState(GL_VERTEX_ARRAY);
-        glVertexPointer_p(3, $self->bed_grid_lines);
+        my $grid_vertex;
+        if (HAS_VBO) {
+            ($grid_vertex) =
+                glGenBuffersARB_p(1);
+            $self->bed_grid_lines->bind($grid_vertex);
+            glBufferDataARB_p(GL_ARRAY_BUFFER_ARB, $self->bed_grid_lines, GL_STATIC_DRAW_ARB);
+            glVertexPointer_c(3, GL_FLOAT, 0, 0);
+        } else {
+            # fall back on old behavior
+            glVertexPointer_c(3, GL_FLOAT, 0, $self->bed_grid_lines->ptr());
+        }
+        glColor4f( @GRID_COLOR );
+        glNormal3d(0,0,1);
         glDrawArrays(GL_LINES, 0, $self->bed_grid_lines->elements / 3);
         glDisableClientState(GL_VERTEX_ARRAY);
         
         glDisable(GL_BLEND);
+        if (HAS_VBO) { 
+            # Turn off buffer objects to let the rest of the draw code work.
+            glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
+            glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
+            glDeleteBuffersARB_p($grid_vertex);
+            glDeleteBuffersARB_p($triangle_vertex);
+        }
     }
     
     my $volumes_bb = $self->volumes_bounding_box;
@@ -770,8 +939,8 @@ sub Render {
         glDisable(GL_DEPTH_TEST);
         my $origin = $self->origin;
         my $axis_len = max(
-            0.3 * max(@{ $self->bed_bounding_box->size }),
-              2 * max(@{ $volumes_bb->size }),
+            max(@{ $self->bed_bounding_box->size }),
+            1.2 * max(@{ $volumes_bb->size }),
         );
         glLineWidth(2);
         glBegin(GL_LINES);
@@ -808,19 +977,69 @@ sub Render {
         glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         glBegin(GL_QUADS);
-        glColor4f(0.8, 0.8, 0.8, 0.5);
-        glVertex3f($bb->x_min-20, $bb->y_min-20, $plane_z);
-        glVertex3f($bb->x_max+20, $bb->y_min-20, $plane_z);
-        glVertex3f($bb->x_max+20, $bb->y_max+20, $plane_z);
-        glVertex3f($bb->x_min-20, $bb->y_max+20, $plane_z);
+        glColor4f( @COLOR_CUTPLANE );
+        if ($self->cutting_plane_axis == X) {
+            glVertex3f($bb->x_min+$plane_z, $bb->y_min-20, $bb->z_min-20);
+            glVertex3f($bb->x_min+$plane_z, $bb->y_max+20, $bb->z_min-20);
+            glVertex3f($bb->x_min+$plane_z, $bb->y_max+20, $bb->z_max+20);
+            glVertex3f($bb->x_min+$plane_z, $bb->y_min-20, $bb->z_max+20);
+        } elsif ($self->cutting_plane_axis == Y) {
+            glVertex3f($bb->x_min-20, $bb->y_min+$plane_z, $bb->z_min-20);
+            glVertex3f($bb->x_max+20, $bb->y_min+$plane_z, $bb->z_min-20);
+            glVertex3f($bb->x_max+20, $bb->y_min+$plane_z, $bb->z_max+20);
+            glVertex3f($bb->x_min-20, $bb->y_min+$plane_z, $bb->z_max+20);
+        } elsif ($self->cutting_plane_axis == Z) {
+            glVertex3f($bb->x_min-20, $bb->y_min-20, $bb->z_min+$plane_z);
+            glVertex3f($bb->x_max+20, $bb->y_min-20, $bb->z_min+$plane_z);
+            glVertex3f($bb->x_max+20, $bb->y_max+20, $bb->z_min+$plane_z);
+            glVertex3f($bb->x_min-20, $bb->y_max+20, $bb->z_min+$plane_z);
+        }
         glEnd();
         glEnable(GL_CULL_FACE);
         glDisable(GL_BLEND);
     }
     
+    if (defined $self->_drag_start_pos || defined $self->_drag_start_xy) {
+        $self->draw_center_of_rotation($self->_camera_target->x, $self->_camera_target->y, $self->_camera_target->z);
+    }
+
     glFlush();
  
     $self->SwapBuffers();
+    
+    # Calling glFinish has a performance penalty, but it seems to fix some OpenGL driver hang-up with extremely large scenes.
+    glFinish();
+}
+
+sub draw_axes {
+    my ($self, $x, $y, $z, $length, $width, $allways_visible) = @_;
+    if ($allways_visible) {
+        glDisable(GL_DEPTH_TEST);
+    } else {
+        glEnable(GL_DEPTH_TEST);
+    }
+    glLineWidth($width);
+    glBegin(GL_LINES);
+    # draw line for x axis
+    glColor3f(1, 0, 0);
+    glVertex3f($x, $y, $z);
+    glVertex3f($x + $length, $y, $z);
+    # draw line for y axis
+    glColor3f(0, 1, 0);
+    glVertex3f($x, $y, $z);
+    glVertex3f($x, $y + $length, $z);
+    # draw line for Z axis
+    glColor3f(0, 0, 1);
+    glVertex3f($x, $y, $z);
+    glVertex3f($x, $y, $z + $length);
+    glEnd();
+}
+
+sub draw_center_of_rotation {
+    my ($self, $x, $y, $z) = @_;
+    
+    $self->draw_axes($x, $y, $z, 10, 1, 1);
+    $self->draw_axes($x, $y, $z, 10, 4, 0);
 }
 
 sub draw_volumes {
@@ -843,9 +1062,9 @@ sub draw_volumes {
             my $b = ($volume_idx & 0x00FF0000) >> 16;
             glColor4f($r/255.0, $g/255.0, $b/255.0, 1);
         } elsif ($volume->selected) {
-            glColor4f(@{ &SELECTED_COLOR });
+            glColor4f( @SELECTED_COLOR , $volume->color->[3]);
         } elsif ($volume->hover) {
-            glColor4f(@{ &HOVER_COLOR });
+            glColor4f( @HOVER_COLOR, $volume->color->[3]);
         } else {
             glColor4f(@{ $volume->color });
         }
@@ -897,12 +1116,30 @@ sub draw_volumes {
     }
     glDisableClientState(GL_NORMAL_ARRAY);
     glDisable(GL_BLEND);
-    
+
+    my $cut_vertex;
     if (defined $self->cutting_plane_z) {
+        if (HAS_VBO) {
+            # Use Vertex Buffer Object for cutting plane (previous method crashes on modern POGL). 
+            ($cut_vertex) = glGenBuffersARB_p(1);
+            $self->cut_lines_vertices->bind($cut_vertex);
+            glBufferDataARB_p(GL_ARRAY_BUFFER_ARB, $self->cut_lines_vertices, GL_STATIC_DRAW_ARB);
+            glVertexPointer_c(3, GL_FLOAT, 0, 0);
+        } else {
+            # Use legacy method.
+            glVertexPointer_c(3, GL_FLOAT, 0, $self->cut_lines_vertices->ptr());
+        }
         glLineWidth(2);
         glColor3f(0, 0, 0);
-        glVertexPointer_p(3, $self->cut_lines_vertices);
         glDrawArrays(GL_LINES, 0, $self->cut_lines_vertices->elements / 3);
+
+        if (HAS_VBO) { 
+            # Turn off buffer objects to let the rest of the draw code work.
+            glBindBufferARB(GL_ARRAY_BUFFER_ARB, 0);
+            glBindBufferARB(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
+            glDeleteBuffersARB_p($cut_vertex);
+        }
+
     }
     glDisableClientState(GL_VERTEX_ARRAY);
 }
@@ -940,22 +1177,35 @@ use OpenGL qw(:glconstants :gluconstants :glufunctions);
 use List::Util qw(first min max);
 use Slic3r::Geometry qw(scale unscale epsilon);
 use Slic3r::Print::State ':steps';
-
-use constant COLORS => [ [1,1,0,1], [1,0.5,0.5,1], [0.5,1,0.5,1], [0.5,0.5,1,1] ];
+use Slic3r::GUI::ColorScheme;
 
 __PACKAGE__->mk_accessors(qw(
+    colors
     color_by
+    color_toolpaths_by
     select_by
     drag_by
     volumes_by_object
     _objects_by_volumes
 ));
 
+sub default_colors { [@COLOR_PARTS], [@COLOR_INFILL], [@COLOR_SUPPORT], [@COLOR_UNKNOWN] }
+
 sub new {
     my $class = shift;
     
     my $self = $class->SUPER::new(@_);
+    
+    if ( ( defined $Slic3r::GUI::Settings->{_}{colorscheme} ) && ( Slic3r::GUI::ColorScheme->can($Slic3r::GUI::Settings->{_}{colorscheme}) ) ) {
+        my $myGetSchemeName = \&{"Slic3r::GUI::ColorScheme::$Slic3r::GUI::Settings->{_}{colorscheme}"};
+        $myGetSchemeName->();
+    } else {
+        Slic3r::GUI::ColorScheme->getDefault();
+    }
+    
+    $self->colors([ $self->default_colors ]);
     $self->color_by('volume');      # object | volume
+    $self->color_toolpaths_by('role'); # role | extruder
     $self->select_by('object');     # object | volume | instance
     $self->drag_by('instance');     # object | instance
     $self->volumes_by_object({});   # obj_idx => [ volume_idx, volume_idx ... ]
@@ -993,13 +1243,12 @@ sub load_object {
                 $color_idx = $obj_idx;
             }
         
-            my $color = [ @{COLORS->[ $color_idx % scalar(@{&COLORS}) ]} ];
+            my $color = [ @{$self->colors->[ $color_idx % scalar(@{$self->colors}) ]} ];
             $color->[3] = $volume->modifier ? 0.5 : 1;
             push @{$self->volumes}, my $v = Slic3r::GUI::3DScene::Volume->new(
                 bounding_box    => $mesh->bounding_box,
                 color           => $color,
             );
-            $v->mesh($mesh) if $self->enable_cutting;
             if ($self->select_by eq 'object') {
                 $v->select_group_id($obj_idx*1000000);
             } elsif ($self->select_by eq 'volume') {
@@ -1071,7 +1320,7 @@ sub load_print_object_slices {
     
     push @{$self->volumes}, my $v = Slic3r::GUI::3DScene::Volume->new(
         bounding_box    => $bb,
-        color           => COLORS->[0],
+        color           => $self->colors->[0],
         verts           => OpenGL::Array->new_list(GL_FLOAT, @verts),
         norms           => OpenGL::Array->new_list(GL_FLOAT, @norms),
         quad_verts      => OpenGL::Array->new_list(GL_FLOAT, @quad_verts),
@@ -1084,38 +1333,10 @@ sub load_print_toolpaths {
     
     return if !$print->step_done(STEP_SKIRT);
     return if !$print->step_done(STEP_BRIM);
-    return if !$print->has_skirt && $print->config->brim_width == 0;
-    
-    my $qverts  = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my $tverts  = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my %offsets = ();  # print_z => [ qverts, tverts ]
-    
-    my $skirt_height = 0;  # number of layers
-    if ($print->has_infinite_skirt) {
-        $skirt_height = $print->total_layer_count;
-    } else {
-        $skirt_height = min($print->config->skirt_height, $print->total_layer_count);
-    }
-    $skirt_height ||= 1 if $print->config->brim_width > 0;
-    
-    # get first $skirt_height layers (maybe this should be moved to a PrintObject method?)
-    my $object0 = $print->get_object(0);
-    my @layers = ();
-    push @layers, map $object0->get_layer($_-1), 1..min($skirt_height, $object0->layer_count);
-    push @layers, map $object0->get_support_layer($_-1), 1..min($skirt_height, $object0->support_layer_count);
-    @layers = sort { $a->print_z <=> $b->print_z } @layers;
-    @layers = @layers[0..($skirt_height-1)];
-    
-    foreach my $i (0..($skirt_height-1)) {
-        my $top_z = $layers[$i]->print_z;
-        $offsets{$top_z} = [$qverts->size, $tverts->size];
-        
-        if ($i == 0) {
-            $self->_extrusionentity_to_verts($print->brim, $top_z, Slic3r::Point->new(0,0), $qverts, $tverts);
-        }
-        
-        $self->_extrusionentity_to_verts($print->skirt, $top_z, Slic3r::Point->new(0,0), $qverts, $tverts);
-    }
+    return if !$print->has_skirt
+        && $print->config->brim_width == 0
+        && $print->config->interior_brim_width == 0
+        && $print->config->brim_connections_width == 0;
     
     my $bb = Slic3r::Geometry::BoundingBoxf3->new;
     {
@@ -1123,103 +1344,170 @@ sub load_print_toolpaths {
         $bb->merge_point(Slic3r::Pointf3->new_unscale(@{$pbb->min_point}));
         $bb->merge_point(Slic3r::Pointf3->new_unscale(@{$pbb->max_point}));
     }
-    push @{$self->volumes}, Slic3r::GUI::3DScene::Volume->new(
-        bounding_box    => $bb,
-        color           => COLORS->[2],
-        qverts          => $qverts,
-        tverts          => $tverts,
-        offsets         => { %offsets },
-    );
+    
+    my $color_by_extruder = $self->color_toolpaths_by eq 'extruder';
+    
+    if (!$print->brim->empty) {
+        my $color = $color_by_extruder
+            ? $self->colors->[ ($print->brim_extruder-1) % @{$self->colors} ]
+            : $self->colors->[2];
+        
+        push @{$self->volumes}, my $volume = Slic3r::GUI::3DScene::Volume->new(
+            bounding_box    => $bb,
+            color           => $color,
+            qverts          => Slic3r::GUI::_3DScene::GLVertexArray->new,
+            tverts          => Slic3r::GUI::_3DScene::GLVertexArray->new,
+            offsets         => {},  # print_z => [ qverts, tverts ]
+        );
+        
+        my $top_z = $print->get_object(0)->get_layer(0)->print_z;
+        $volume->offsets->{$top_z} = [0, 0];
+        $self->_extrusionentity_to_verts($print->brim, $top_z, Slic3r::Point->new(0,0),
+            $volume->qverts, $volume->tverts);
+    }
+    
+    if (!$print->skirt->empty) {
+        # TODO: it's a bit difficult to predict skirt extruders with the current skirt logic.
+        # We need to rewrite it anyway.
+        my $color = +($self->default_colors)[0];
+        push @{$self->volumes}, my $volume = Slic3r::GUI::3DScene::Volume->new(
+            bounding_box    => $bb,
+            color           => $color,
+            qverts          => Slic3r::GUI::_3DScene::GLVertexArray->new,
+            tverts          => Slic3r::GUI::_3DScene::GLVertexArray->new,
+            offsets         => {},  # print_z => [ qverts, tverts ]
+        );
+    
+        my $skirt_height = 0;  # number of layers
+        if ($print->has_infinite_skirt) {
+            $skirt_height = $print->total_layer_count;
+        } else {
+            $skirt_height = min($print->config->skirt_height, $print->total_layer_count);
+        }
+        $skirt_height ||= 1 if $print->config->brim_width > 0 || $print->config->interior_brim_width;
+    
+        # get first $skirt_height layers (maybe this should be moved to a PrintObject method?)
+        my $object0 = $print->get_object(0);
+        my @layers = ();
+        push @layers, map $object0->get_layer($_-1), 1..min($skirt_height, $object0->layer_count);
+        push @layers, map $object0->get_support_layer($_-1), 1..min($skirt_height, $object0->support_layer_count);
+        @layers = sort { $a->print_z <=> $b->print_z } @layers;
+        @layers = @layers[0..($skirt_height-1)];
+    
+        foreach my $i (0..($skirt_height-1)) {
+            my $top_z = $layers[$i]->print_z;
+            $volume->offsets->{$top_z} = [$volume->qverts->size, $volume->tverts->size];
+        
+            $self->_extrusionentity_to_verts($print->skirt, $top_z, Slic3r::Point->new(0,0),
+                $volume->qverts, $volume->tverts);
+        }
+    }
 }
 
 sub load_print_object_toolpaths {
     my ($self, $object) = @_;
     
-    my $perim_qverts    = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my $perim_tverts    = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my $infill_qverts   = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my $infill_tverts   = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my $support_qverts  = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    my $support_tverts  = Slic3r::GUI::_3DScene::GLVertexArray->new;
-    
-    my %perim_offsets   = ();  # print_z => [ qverts, tverts ]
-    my %infill_offsets  = ();
-    my %support_offsets = ();
-    
     # order layers by print_z
     my @layers = sort { $a->print_z <=> $b->print_z }
         @{$object->layers}, @{$object->support_layers};
     
-    foreach my $layer (@layers) {
-        my $top_z = $layer->print_z;
+    # Bounding box of the object and its copies.
+    my $bb = Slic3r::Geometry::BoundingBoxf3->new;
+    {
+        my $obb = $object->bounding_box;
+        foreach my $copy (@{ $object->_shifted_copies }) {
+            my $cbb = $obb->clone;
+            $cbb->translate(@$copy);
+            $bb->merge_point(Slic3r::Pointf3->new_unscale(@{$cbb->min_point}, 0));
+            $bb->merge_point(Slic3r::Pointf3->new_unscale(@{$cbb->max_point}, $object->size->z));
+        }
+    }
+    
+    my %volumes = ();  # color => Volume
+    
+    # Maximum size of an allocation block: 32MB / sizeof(float)
+    my $alloc_size_max = 32 * 1048576 / 4;
+    
+    my $add = sub {
+        my ($coll, $top_z, $copy, $color) = @_;
         
-        if (!exists $perim_offsets{$top_z}) {
-            $perim_offsets{$top_z} = [
-                $perim_qverts->size, $perim_tverts->size,
-            ];
-            $infill_offsets{$top_z} = [
-                $infill_qverts->size, $infill_tverts->size,
-            ];
-            $support_offsets{$top_z} = [
-                $support_qverts->size, $support_tverts->size,
-            ];
+        my $volume = $volumes{$color};
+        if (!$volume) {
+            push @{$self->volumes}, $volumes{$color} = $volume = Slic3r::GUI::3DScene::Volume->new(
+                bounding_box    => $bb,
+                color           => $color,
+                qverts          => Slic3r::GUI::_3DScene::GLVertexArray->new,
+                tverts          => Slic3r::GUI::_3DScene::GLVertexArray->new,
+                offsets         => {},  # print_z => [ qverts, tverts ]
+            );
         }
         
+        $volume->offsets->{$top_z} //= [ $volume->qverts->size, $volume->tverts->size ];
+        $self->_extrusionentity_to_verts($coll, $top_z, $copy, $volume->qverts, $volume->tverts);
+    };
+    
+    my $color_by_extruder = $self->color_toolpaths_by eq 'extruder';
+    
+    foreach my $layer (@layers) {
+        my $top_z = $layer->print_z;
         foreach my $copy (@{ $object->_shifted_copies }) {
             foreach my $layerm (@{$layer->regions}) {
                 if ($object->step_done(STEP_PERIMETERS)) {
-                    $self->_extrusionentity_to_verts($layerm->perimeters, $top_z, $copy,
-                        $perim_qverts, $perim_tverts);
+                    my $color = $color_by_extruder
+                        ? $self->colors->[ ($layerm->region->config->perimeter_extruder-1) % @{$self->colors} ]
+                        : $self->colors->[0];
+                    $add->($layerm->perimeters, $top_z, $copy, $color);
                 }
                 
                 if ($object->step_done(STEP_INFILL)) {
-                    $self->_extrusionentity_to_verts($layerm->fills, $top_z, $copy,
-                        $infill_qverts, $infill_tverts);
+                    my $color = $color_by_extruder
+                        ? $self->colors->[ ($layerm->region->config->infill_extruder-1) % @{$self->colors} ]
+                        : $self->colors->[1];
+                    
+                    if ($color_by_extruder && $layerm->region->config->infill_extruder != $layerm->region->config->solid_infill_extruder) {
+                        # divide solid and non-solid infill
+                        my $solid = Slic3r::ExtrusionPath::Collection->new;
+                        my $non_solid = Slic3r::ExtrusionPath::Collection->new;
+                        foreach my $fill (@{$layerm->fills}) {
+                            if ($fill->[0]->is_solid_infill) {
+                                $solid->append($fill);
+                            } else {
+                                $non_solid->append($fill);
+                            }
+                        }
+                        $add->($non_solid, $top_z, $copy, $color);
+                        $color = $self->colors->[ ($layerm->region->config->solid_infill_extruder-1) % @{$self->colors} ];
+                        $add->($solid, $top_z, $copy, $color);
+                    } else {
+                        $add->($layerm->fills, $top_z, $copy, $color);
+                    }
                 }
             }
             
             if ($layer->isa('Slic3r::Layer::Support') && $object->step_done(STEP_SUPPORTMATERIAL)) {
-                $self->_extrusionentity_to_verts($layer->support_fills, $top_z, $copy,
-                    $support_qverts, $support_tverts);
-                
-                $self->_extrusionentity_to_verts($layer->support_interface_fills, $top_z, $copy,
-                    $support_qverts, $support_tverts);
+                {
+                    my $color = $color_by_extruder
+                        ? $self->colors->[ ($layer->object->config->support_material_extruder-1) % @{$self->colors} ]
+                        : $self->colors->[2];
+                    $add->($layer->support_fills, $top_z, $copy, $color);
+                }
+                {
+                    my $color = ($color_by_extruder)
+                        ? $self->colors->[ ($layer->object->config->support_material_interface_extruder-1) % @{$self->colors} ]
+                        : $self->colors->[2];
+                    $add->($layer->support_interface_fills, $top_z, $copy, $color);
+                }
+            }
+        }
+        
+        foreach my $color (keys %volumes) {
+            my $volume = $volumes{$color};
+            if ($volume->qverts->size() > $alloc_size_max || $volume->tverts->size() > $alloc_size_max) {
+                # stop appending to this volume; create a new one next time
+                delete $volumes{$color};
             }
         }
     }
-    
-    my $obb = $object->bounding_box;
-    my $bb = Slic3r::Geometry::BoundingBoxf3->new;
-    foreach my $copy (@{ $object->_shifted_copies }) {
-        my $cbb = $obb->clone;
-        $cbb->translate(@$copy);
-        $bb->merge_point(Slic3r::Pointf3->new_unscale(@{$cbb->min_point}, 0));
-        $bb->merge_point(Slic3r::Pointf3->new_unscale(@{$cbb->max_point}, $object->size->z));
-    }
-    
-    push @{$self->volumes}, Slic3r::GUI::3DScene::Volume->new(
-        bounding_box    => $bb,
-        color           => COLORS->[0],
-        qverts          => $perim_qverts,
-        tverts          => $perim_tverts,
-        offsets         => { %perim_offsets },
-    );
-    
-    push @{$self->volumes}, Slic3r::GUI::3DScene::Volume->new(
-        bounding_box    => $bb,
-        color           => COLORS->[1],
-        qverts          => $infill_qverts,
-        tverts          => $infill_tverts,
-        offsets         => { %infill_offsets },
-    );
-    
-    push @{$self->volumes}, Slic3r::GUI::3DScene::Volume->new(
-        bounding_box    => $bb,
-        color           => COLORS->[2],
-        qverts          => $support_qverts,
-        tverts          => $support_tverts,
-        offsets         => { %support_offsets },
-    );
 }
 
 sub set_toolpaths_range {
@@ -1258,6 +1546,8 @@ sub _expolygons_to_verts {
     gluDeleteTess($tess);
 }
 
+# Fill in the $qverts and $tverts with quads and triangles
+# for the extrusion $entity.
 sub _extrusionentity_to_verts {
     my ($self, $entity, $top_z, $copy, $qverts, $tverts) = @_;
     
@@ -1289,8 +1579,18 @@ sub _extrusionentity_to_verts {
             push @$heights, map $path->height, 0..$#$path_lines;
         }
     }
+    
+    # Calling the C++ implementation Slic3r::_3DScene::_extrusionentity_to_verts_do()
+    # This adds new vertices to the $qverts and $tverts.
     Slic3r::GUI::_3DScene::_extrusionentity_to_verts_do($lines, $widths, $heights,
-        $closed, $top_z, $copy, $qverts, $tverts);
+        $closed, 
+        # Top height of the extrusion.
+        $top_z, 
+        # $copy is not used here.
+        $copy,
+        # GLVertexArray object: C++ class maintaining an std::vector<float> for coords and normals.
+        $qverts,
+        $tverts);
 }
 
 sub object_idx {
