@@ -1,3 +1,7 @@
+# 2D preview of the tool paths of a single layer, using a thin line.
+# OpenGL is used to render the paths.
+# Vojtech also added a 2D simulation of under/over extrusion in a single layer.
+
 package Slic3r::GUI::Plater::2DToolpaths;
 use strict;
 use warnings;
@@ -8,15 +12,24 @@ use Wx qw(:misc :sizer :slider :statictext wxWHITE);
 use Wx::Event qw(EVT_SLIDER EVT_KEY_DOWN);
 use base qw(Wx::Panel Class::Accessor);
 
+# Color Scheme
+use Slic3r::GUI::ColorScheme;
+
 __PACKAGE__->mk_accessors(qw(print enabled));
 
 sub new {
     my $class = shift;
     my ($parent, $print) = @_;
-    
+
     my $self = $class->SUPER::new($parent, -1, wxDefaultPosition);
-    $self->SetBackgroundColour(wxWHITE);
-    
+    if ( ( defined $Slic3r::GUI::Settings->{_}{colorscheme} ) && ( my $getScheme =  Slic3r::GUI::ColorScheme->can($Slic3r::GUI::Settings->{_}{colorscheme}) ) ) {
+        $getScheme->();
+        $self->SetBackgroundColour(Wx::Colour->new(@BACKGROUND255));
+    } else {
+        Slic3r::GUI::ColorScheme->getDefault();
+        $self->SetBackgroundColour(Wx::wxWHITE);
+    }
+
     # init GUI elements
     my $canvas = $self->{canvas} = Slic3r::GUI::Plater::2DToolpaths::Canvas->new($self, $print);
     my $slider = $self->{slider} = Wx::Slider->new(
@@ -56,6 +69,8 @@ sub new {
         } elsif ($key == 68 || $key == 317) {
             $slider->SetValue($slider->GetValue - 1);
             $self->set_z($self->{layers_z}[$slider->GetValue]);
+        } else {
+            $event->Skip;
         }
     });
     
@@ -124,6 +139,9 @@ use List::Util qw(min max first);
 use Slic3r::Geometry qw(scale unscale epsilon X Y);
 use Slic3r::Print::State ':steps';
 
+# Color Scheme
+use Slic3r::GUI::ColorScheme;
+
 __PACKAGE__->mk_accessors(qw(
     print z layers color init
     bb
@@ -142,11 +160,23 @@ __PACKAGE__->mk_accessors(qw(
 
 sub new {
     my ($class, $parent, $print) = @_;
-    
-    my $self = $class->SUPER::new($parent);
+
+    if ( ( defined $Slic3r::GUI::Settings->{_}{colorscheme} ) && ( Slic3r::GUI::ColorScheme->can($Slic3r::GUI::Settings->{_}{colorscheme}) ) ) {
+        my $myGetSchemeName = \&{"Slic3r::GUI::ColorScheme::$Slic3r::GUI::Settings->{_}{colorscheme}"};
+        $myGetSchemeName->();
+    } else {
+        Slic3r::GUI::ColorScheme->getDefault();
+    }
+    my $self = (Wx::wxVERSION >= 3.000003) ?
+        # The wxWidgets 3.0.3-beta have a bug, they crash with NULL attribute list.
+        $class->SUPER::new($parent, -1, Wx::wxDefaultPosition, Wx::wxDefaultSize, 0, "",
+            [WX_GL_RGBA, WX_GL_DOUBLEBUFFER, WX_GL_DEPTH_SIZE, 24, 0]) :
+        $class->SUPER::new($parent);
+    # Immediatelly force creation of the OpenGL context to consume the static variable s_wglContextAttribs.
+    $self->GetContext();
     $self->print($print);
     $self->_zoom(1);
-    
+
     # 2D point in model space
     $self->_camera_target(Slic3r::Pointf->new(0,0));
     
@@ -164,29 +194,34 @@ sub new {
     EVT_MOUSEWHEEL($self, sub {
         my ($self, $e) = @_;
         
+        return if !$self->GetParent->enabled;
+        
         my $old_zoom = $self->_zoom;
         
         # Calculate the zoom delta and apply it to the current zoom factor
-        my $zoom = $e->GetWheelRotation() / $e->GetWheelDelta();
+        my $zoom = -$e->GetWheelRotation() / $e->GetWheelDelta();
+        if ($Slic3r::GUI::Settings->{_}{invert_zoom}) {
+            $zoom *= -1;
+        }
         $zoom = max(min($zoom, 4), -4);
         $zoom /= 10;
         $self->_zoom($self->_zoom / (1-$zoom));
         $self->_zoom(1) if $self->_zoom > 1;  # prevent from zooming out too much
-        
+
         {
             # In order to zoom around the mouse point we need to translate
             # the camera target. This math is almost there but not perfect yet...
             my $camera_bb_size = $self->_camera_bb->size;
             my $size = Slic3r::Pointf->new($self->GetSizeWH);
             my $pos = Slic3r::Pointf->new($e->GetPositionXY);
-            
+
             # calculate the zooming center in pixel coordinates relative to the viewport center
             my $vec = Slic3r::Pointf->new($pos->x - $size->x/2, $pos->y - $size->y/2);  #-
-            
+
             # calculate where this point will end up after applying the new zoom
             my $vec2 = $vec->clone;
             $vec2->scale($old_zoom / $self->_zoom);
-            
+
             # move the camera target by the difference of the two positions
             $self->_camera_target->translate(
                 -($vec->x - $vec2->x) * $camera_bb_size->x / $size->x,
@@ -202,9 +237,25 @@ sub new {
     return $self;
 }
 
+sub zoom{
+    my($self, $direction) = @_;
+    if( $direction eq 'in'){
+        $self->_zoom($self->_zoom / (1+0.3));
+    }
+    elsif($direction eq 'out'){
+        $self->_zoom($self->_zoom / (1-0.3));
+        $self->_zoom(1) if $self->_zoom > 1;  # prevent from zooming out too much
+    }
+    #apply changes
+    $self->_dirty(1);
+    $self->Refresh;
+}
+
 sub mouse_event {
     my ($self, $e) = @_;
     
+    return if !$self->GetParent->enabled;
+    
     my $pos = Slic3r::Pointf->new($e->GetPositionXY);
     if ($e->Entering && &Wx::wxMSW) {
         # wxMSW needs focus in order to catch mouse wheel events
@@ -277,7 +328,12 @@ sub Render {
     $self->SetCurrent($context);
     $self->InitGL;
     
-    glClearColor(1, 1, 1, 0);
+    if ($DEFAULT_COLORSCHEME==1){
+		glClearColor(1, 1, 1, 0);
+	} else{
+		glClearColor(@BACKGROUND_COLOR, 0);
+	}
+    
     glClear(GL_COLOR_BUFFER_BIT);
     
     if (!$self->GetParent->enabled || !$self->layers) {
@@ -325,7 +381,7 @@ sub Render {
             glTranslatef(@$copy, 0);
             
             foreach my $slice (@{$layer->slices}) {
-                glColor3f(0.95, 0.95, 0.95);
+                glColor3f(@TOOL_SHADE); # Inside part shade
                 
                 if ($tess) {
                     gluTessBeginPolygon($tess);
@@ -337,7 +393,7 @@ sub Render {
                     gluTessEndPolygon($tess);
                 }
                 
-                glColor3f(0.9, 0.9, 0.9);
+                glColor3f(@TOOL_COLOR); # Perimeter
                 foreach my $polygon (@$slice) {
                     foreach my $line (@{$polygon->lines}) {
                         glBegin(GL_LINES);
@@ -359,33 +415,33 @@ sub Render {
         
         # draw brim
         if ($self->print->step_done(STEP_BRIM) && $layer->id == 0 && !$brim_drawn) {
-            $self->color([0, 0, 0]);
+            $self->color(@TOOL_DARK);
             $self->_draw(undef, $print_z, $_) for @{$self->print->brim};
             $brim_drawn = 1;
         }
         if ($self->print->step_done(STEP_SKIRT)
             && ($self->print->has_infinite_skirt() || $self->print->config->skirt_height > $layer->id)
             && !$skirt_drawn) {
-            $self->color([0, 0, 0]);
+            $self->color(@TOOL_DARK);
             $self->_draw(undef, $print_z, $_) for @{$self->print->skirt};
             $skirt_drawn = 1;
         }
         
         foreach my $layerm (@{$layer->regions}) {
             if ($object->step_done(STEP_PERIMETERS)) {
-                $self->color([0.7, 0, 0]);
+                $self->color(@TOOL_STEPPERIM);
                 $self->_draw($object, $print_z, $_) for map @$_, @{$layerm->perimeters};
             }
             
             if ($object->step_done(STEP_INFILL)) {
-                $self->color([0, 0, 0.7]);
+                $self->color(@TOOL_INFILL);
                 $self->_draw($object, $print_z, $_) for map @$_, @{$layerm->fills};
             }
         }
         
         if ($object->step_done(STEP_SUPPORTMATERIAL)) {
             if ($layer->isa('Slic3r::Layer::Support')) {
-                $self->color([0, 0, 0]);
+                $self->color(@TOOL_SUPPORT);
                 $self->_draw($object, $print_z, $_) for @{$layer->support_fills};
                 $self->_draw($object, $print_z, $_) for @{$layer->support_interface_fills};
             }
@@ -413,7 +469,7 @@ sub _draw_path {
     return if $print_z - $path->height > $self->z - epsilon;
     
     if (abs($print_z - $self->z) < epsilon) {
-        glColor3f(@{$self->color});
+        glColor3f($self->color->[0], $self->color->[1], $self->color->[2]);
     } else {
         glColor3f(0.8, 0.8, 0.8);
     }
