@@ -1,3 +1,5 @@
+# The slicing work horse.
+# Extends C++ class Slic3r::Print
 package Slic3r::Print;
 use strict;
 use warnings;
@@ -5,25 +7,17 @@ use warnings;
 use File::Basename qw(basename fileparse);
 use File::Spec;
 use List::Util qw(min max first sum);
+use Slic3r::ExtrusionLoop ':roles';
 use Slic3r::ExtrusionPath ':roles';
 use Slic3r::Flow ':roles';
 use Slic3r::Geometry qw(X Y Z X1 Y1 X2 Y2 MIN MAX PI scale unscale convex_hull);
 use Slic3r::Geometry::Clipper qw(diff_ex union_ex intersection_ex intersection offset
-    offset2 union union_pt_chained JT_ROUND JT_SQUARE);
+    offset2 union union_pt_chained JT_ROUND JT_SQUARE diff_pl);
 use Slic3r::Print::State ':steps';
+use Slic3r::Surface qw(S_TYPE_BOTTOM);
 
 our $status_cb;
 
-sub new {
-    # TODO: port PlaceholderParser methods to C++, then its own constructor
-    # can call them and no need for this new() method at all
-    my ($class) = @_;
-    my $self = $class->_new;
-    $self->placeholder_parser->apply_env_variables;
-    $self->placeholder_parser->update_timestamp;
-    return $self;
-}
-
 sub set_status_cb {
     my ($class, $cb) = @_;
     $status_cb = $cb;
@@ -48,8 +42,14 @@ sub size {
 sub process {
     my ($self) = @_;
     
-    $_->make_perimeters for @{$self->objects};
+    ### No need to call this as we call it as part of prepare_infill()
+    ### until we fix the idempotency issue.
+    ###$self->status_cb->(20, "Generating perimeters");
+    ###$_->make_perimeters for @{$self->objects};
+    
+    $self->status_cb->(70, "Infilling layers");
     $_->infill for @{$self->objects};
+    
     $_->generate_support_material for @{$self->objects};
     $self->make_skirt;
     $self->make_brim;  # must come after make_skirt
@@ -71,6 +71,34 @@ sub process {
     }
 }
 
+sub escaped_split {
+    my ($line) = @_;
+
+    # Free up three characters for temporary replacement
+    $line =~ s/%/%%/g;
+    $line =~ s/#/##/g;
+    $line =~ s/\?/\?\?/g;
+
+    # replace escaped !'s
+    $line =~ s/\!\!/%#\?/g;
+    
+    # split on non-escaped whitespace
+    my @split = split /(?<=[^\!])\s+/, $line, -1;
+
+    for my $part (@split) {
+      # replace escaped whitespace with the whitespace
+      $part =~ s/\!(\s+)/$1/g;
+
+      # resub temp symbols
+      $part =~ s/%#\?/\!/g;
+      $part =~ s/%%/%/g;
+      $part =~ s/##/#/g;
+      $part =~ s/\?\?/\?/g;
+    }
+
+    return @split;
+}
+
 sub export_gcode {
     my $self = shift;
     my %params = @_;
@@ -79,9 +107,41 @@ sub export_gcode {
     $self->process;
     
     # output everything to a G-code file
-    my $output_file = $self->expanded_output_filepath($params{output_file});
+    my $output_file = $self->output_filepath($params{output_file} // '');
     $self->status_cb->(90, "Exporting G-code" . ($output_file ? " to $output_file" : ""));
-    $self->write_gcode($params{output_fh} || $output_file);
+    
+    {
+        # open output gcode file if we weren't supplied a file-handle
+        my ($fh, $tempfile);
+        if ($params{output_fh}) {
+            $fh = $params{output_fh};
+        } else {
+            $tempfile = "$output_file.tmp";
+            Slic3r::open(\$fh, ">", $tempfile)
+                or die "Failed to open $tempfile for writing\n";
+    
+            # enable UTF-8 output since user might have entered Unicode characters in fields like notes
+            binmode $fh, ':utf8';
+        }
+
+        Slic3r::Print::GCode->new(
+            print   => $self,
+            fh      => $fh,
+        )->export;
+
+        # close our gcode file
+        close $fh;
+        if ($tempfile) {
+            my $renamed = 0;
+            for my $i (1..5) {
+                last if $renamed = rename Slic3r::encode_path($tempfile), Slic3r::encode_path($output_file);
+                # Wait for 1/4 seconds and try to rename once again.
+                select(undef, undef, undef, 0.25);
+            }
+            Slic3r::debugf "Failed to remove the output G-code file from $tempfile to $output_file. Is $tempfile locked?\n"
+                if !$renamed;
+        }
+    }
     
     # run post-processing scripts
     if (@{$self->config->post_process}) {
@@ -89,15 +149,19 @@ sub export_gcode {
         $self->config->setenv;
         for my $script (@{$self->config->post_process}) {
             Slic3r::debugf "  '%s' '%s'\n", $script, $output_file;
+            my @parsed_script = escaped_split $script;
+            my $executable = shift @parsed_script ;
+            push @parsed_script, $output_file;
             # -x doesn't return true on Windows except for .exe files
-            if (($^O eq 'MSWin32') ? !(-e $script) : !(-x $script)) {
-                die "The configured post-processing script is not executable: check permissions. ($script)\n";
+            if (($^O eq 'MSWin32') ? !(-e $executable) : !(-x $executable)) {
+                die "The configured post-processing script is not executable: check permissions or escape whitespace/exclamation points. ($executable) \n";
             }
-            system($script, $output_file);
+            system($executable, @parsed_script);
         }
     }
 }
 
+# Export SVG slices for the offline SLA printing.
 sub export_svg {
     my $self = shift;
     my %params = @_;
@@ -106,7 +170,7 @@ sub export_svg {
     
     my $fh = $params{output_fh};
     if (!$fh) {
-        my $output_file = $self->expanded_output_filepath($params{output_file});
+        my $output_file = $self->output_filepath($params{output_file});
         $output_file =~ s/\.gcode$/.svg/i;
         Slic3r::open(\$fh, ">", $output_file) or die "Failed to open $output_file for writing\n";
         print "Exporting to $output_file..." unless $params{quiet};
@@ -283,7 +347,7 @@ sub make_skirt {
     for (my $i = $skirts; $i > 0; $i--) {
         $distance += scale $spacing;
         my $loop = offset([$convex_hull], $distance, 1, JT_ROUND, scale(0.1))->[0];
-        $self->skirt->append(Slic3r::ExtrusionLoop->new_from_paths(
+        my $eloop = Slic3r::ExtrusionLoop->new_from_paths(
             Slic3r::ExtrusionPath->new(
                 polyline        => Slic3r::Polygon->new(@$loop)->split_at_first_point,
                 role            => EXTR_ROLE_SKIRT,
@@ -291,13 +355,15 @@ sub make_skirt {
                 width           => $flow->width,
                 height          => $first_layer_height, # this will be overridden at G-code export time
             ),
-        ));
+        );
+        $eloop->role(EXTRL_ROLE_SKIRT);
+        $self->skirt->append($eloop);
         
         if ($self->config->min_skirt_length > 0) {
             $extruded_length[$extruder_idx] ||= 0;
             if (!$extruders_e_per_mm[$extruder_idx]) {
                 my $config = Slic3r::Config::GCode->new;
-                $config->apply_print_config($self->config);
+                $config->apply_static($self->config);
                 my $extruder = Slic3r::Extruder->new($extruder_idx, $config);
                 $extruders_e_per_mm[$extruder_idx] = $extruder->e_per_mm($mm3_per_mm);
             }
@@ -326,150 +392,17 @@ sub make_brim {
     $_->generate_support_material for @{$self->objects};
     $self->make_skirt;
     
-    return if $self->step_done(STEP_BRIM);
-    $self->set_step_started(STEP_BRIM);
-    
-    # since this method must be idempotent, we clear brim paths *before*
-    # checking whether we need to generate them
-    $self->brim->clear;
-    
-    if ($self->config->brim_width == 0) {
-        $self->set_step_done(STEP_BRIM);
-        return;
-    }
     $self->status_cb->(88, "Generating brim");
-    
-    # brim is only printed on first layer and uses perimeter extruder
-    my $first_layer_height = $self->skirt_first_layer_height;
-    my $flow = $self->brim_flow;
-    my $mm3_per_mm = $flow->mm3_per_mm;
-    
-    my $grow_distance = $flow->scaled_width / 2;
-    my @islands = (); # array of polygons
-    foreach my $obj_idx (0 .. ($self->object_count - 1)) {
-        my $object = $self->objects->[$obj_idx];
-        my $layer0 = $object->get_layer(0);
-        my @object_islands = (
-            (map $_->contour, @{$layer0->slices}),
-        );
-        if (@{ $object->support_layers }) {
-            my $support_layer0 = $object->support_layers->[0];
-            push @object_islands,
-                (map @{$_->polyline->grow($grow_distance)}, @{$support_layer0->support_fills})
-                if $support_layer0->support_fills;
-            push @object_islands,
-                (map @{$_->polyline->grow($grow_distance)}, @{$support_layer0->support_interface_fills})
-                if $support_layer0->support_interface_fills;
-        }
-        foreach my $copy (@{$object->_shifted_copies}) {
-            push @islands, map { $_->translate(@$copy); $_ } map $_->clone, @object_islands;
-        }
-    }
-    
-    my @loops = ();
-    my $num_loops = sprintf "%.0f", $self->config->brim_width / $flow->width;
-    for my $i (reverse 1 .. $num_loops) {
-        # JT_SQUARE ensures no vertex is outside the given offset distance
-        # -0.5 because islands are not represented by their centerlines
-        # (first offset more, then step back - reverse order than the one used for 
-        # perimeters because here we're offsetting outwards)
-        push @loops, @{offset2(\@islands, ($i + 0.5) * $flow->scaled_spacing, -1.0 * $flow->scaled_spacing, 100000, JT_SQUARE)};
-    }
-    
-    $self->brim->append(map Slic3r::ExtrusionLoop->new_from_paths(
-        Slic3r::ExtrusionPath->new(
-            polyline        => Slic3r::Polygon->new(@$_)->split_at_first_point,
-            role            => EXTR_ROLE_SKIRT,
-            mm3_per_mm      => $mm3_per_mm,
-            width           => $flow->width,
-            height          => $first_layer_height,
-        ),
-    ), reverse @{union_pt_chained(\@loops)});
-    
-    $self->set_step_done(STEP_BRIM);
+    $self->_make_brim;
 }
 
-sub write_gcode {
+# Wrapper around the C++ Slic3r::Print::validate()
+# to produce a Perl exception without a hang-up on some Strawberry perls.
+sub validate
+{
     my $self = shift;
-    my ($file) = @_;
-    
-    # open output gcode file if we weren't supplied a file-handle
-    my $fh;
-    if (ref $file eq 'IO::Scalar') {
-        $fh = $file;
-    } else {
-        Slic3r::open(\$fh, ">", $file)
-            or die "Failed to open $file for writing\n";
-        
-        # enable UTF-8 output since user might have entered Unicode characters in fields like notes
-        binmode $fh, ':utf8';
-    }
-    
-    my $exporter = Slic3r::Print::GCode->new(
-        print   => $self,
-        fh      => $fh,
-    );
-    $exporter->export;
-    
-    # close our gcode file
-    close $fh;
-}
-
-# this method will return the supplied input file path after expanding its
-# format variables with their values
-sub expanded_output_filepath {
-    my $self = shift;
-    my ($path) = @_;
-    
-    return undef if !@{$self->objects};
-    my $input_file = first { defined $_ } map $_->model_object->input_file, @{$self->objects};
-    return undef if !defined $input_file;
-    
-    my $filename = my $filename_base = basename($input_file);
-    $filename_base =~ s/\.[^.]+$//;  # without suffix
-    
-    # set filename in placeholder parser so that it's available also in custom G-code
-    $self->placeholder_parser->set(input_filename => $filename);
-    $self->placeholder_parser->set(input_filename_base => $filename_base);
-    
-    # set other variables from model object
-    $self->placeholder_parser->set_multiple(
-        scale => [ map $_->model_object->instances->[0]->scaling_factor * 100 . "%", @{$self->objects} ],
-    );
-    
-    if ($path && -d $path) {
-        # if output path is an existing directory, we take that and append
-        # the specified filename format
-        $path = File::Spec->join($path, $self->config->output_filename_format);
-    } elsif (!$path) {
-        # if no explicit output file was defined, we take the input
-        # file directory and append the specified filename format
-        $path = (fileparse($input_file))[1] . $self->config->output_filename_format;
-    } else {
-        # path is a full path to a file so we use it as it is
-    }
-    
-    # make sure we use an up-to-date timestamp
-    $self->placeholder_parser->update_timestamp;
-    return $self->placeholder_parser->process($path);
-}
-
-# This method assigns extruders to the volumes having a material
-# but not having extruders set in the volume config.
-sub auto_assign_extruders {
-    my ($self, $model_object) = @_;
-    
-    # only assign extruders if object has more than one volume
-    return if @{$model_object->volumes} == 1;
-    
-    my $extruders = scalar @{ $self->config->nozzle_diameter };
-    foreach my $i (0..$#{$model_object->volumes}) {
-        my $volume = $model_object->volumes->[$i];
-        if ($volume->material_id ne '') {
-            my $extruder_id = $i + 1;
-            $volume->config->set_ifndef('extruder', $extruder_id);
-        }
-    }
+    my $err = $self->_validate;
+    die $err . "\n" if (defined($err) && $err ne '');
 }
 
 1;
