@@ -1,3 +1,5 @@
+# Instantiated by Slic3r::Print::Object->_support_material()
+# only generate() and contact_distance() are called from the outside of this module.
 package Slic3r::Print::SupportMaterial;
 use Moo;
 
@@ -6,7 +8,7 @@ use Slic3r::ExtrusionPath ':roles';
 use Slic3r::Flow ':roles';
 use Slic3r::Geometry qw(epsilon scale scaled_epsilon PI rad2deg deg2rad convex_hull);
 use Slic3r::Geometry::Clipper qw(offset diff union union_ex intersection offset_ex offset2
-    intersection_pl offset2_ex diff_pl);
+    intersection_pl offset2_ex diff_pl diff_ex);
 use Slic3r::Surface ':types';
 
 has 'print_config'      => (is => 'rw', required => 1);
@@ -25,6 +27,7 @@ use constant PILLAR_SIZE    => 2.5;
 use constant PILLAR_SPACING => 10;
 
 sub generate {
+    # $object is Slic3r::Print::Object
     my ($self, $object) = @_;
     
     # Determine the top surfaces of the support, defined as:
@@ -38,7 +41,7 @@ sub generate {
     # the layer heights of support material and to clip support to the object
     # silhouette.
     my ($top) = $self->object_top($object, $contact);
-    
+
     # We now know the upper and lower boundaries for our support material object
     # (@$contact_z and @$top_z), so we can generate intermediate layers.
     my $support_z = $self->support_layers_z(
@@ -88,15 +91,25 @@ sub generate {
 }
 
 sub contact_area {
+    # $object is Slic3r::Print::Object
     my ($self, $object) = @_;
+    my $conf = $self->object_config;
     
     # if user specified a custom angle threshold, convert it to radians
     my $threshold_rad;
-    if ($self->object_config->support_material_threshold) {
-        $threshold_rad = deg2rad($self->object_config->support_material_threshold + 1);  # +1 makes the threshold inclusive
+    if (!($conf->support_material_threshold =~ /%$/)) {
+        $threshold_rad = deg2rad($conf->support_material_threshold + 1);  # +1 makes the threshold inclusive
         Slic3r::debugf "Threshold angle = %d°\n", rad2deg($threshold_rad);
     }
     
+    # Build support on a build plate only? If so, then collect top surfaces into $buildplate_only_top_surfaces
+    # and subtract $buildplate_only_top_surfaces from the contact surfaces, so
+    # there is no contact surface supported by a top surface.
+    my $buildplate_only =
+        ( $conf->support_material || $conf->support_material_enforce_layers)
+        && $conf->support_material_buildplate_only;
+    my $buildplate_only_top_surfaces = [];
+
     # determine contact areas
     my %contact  = ();  # contact_z => [ polygons ]
     my %overhang = ();  # contact_z => [ polygons ] - this stores the actual overhang supported by each contact layer
@@ -105,15 +118,39 @@ sub contact_area {
         # so $layer_id == 0 means first object layer
         # and $layer->id == 0 means first print layer (including raft)
         
-        if ($self->object_config->raft_layers == 0) {
-            next if $layer_id == 0;
-        } elsif (!$self->object_config->support_material) {
+        # if no raft, and we're at layer 0, skip to layer 1
+        if ( $conf->raft_layers == 0 && $layer_id == 0 ) {
+            next;
+        }
+        # with or without raft, if we're above layer 1, we need to quit
+        # support generation if supports are disabled, or if we're at a high
+        # enough layer that enforce-supports no longer applies
+        if ( $layer_id > 0
+              && !$conf->support_material
+              && ($layer_id >= $conf->support_material_enforce_layers) ) {
             # if we are only going to generate raft just check 
             # the 'overhangs' of the first object layer
-            last if $layer_id > 0;
+            last;
         }
         my $layer = $object->get_layer($layer_id);
-        
+		last if $conf->support_material_max_layers
+			&& $layer_id > $conf->support_material_max_layers;
+
+        if ($buildplate_only) {
+            # Collect the top surfaces up to this layer and merge them.
+            my $projection_new = [];
+            push @$projection_new, ( map $_->p, map @{$_->slices->filter_by_type(S_TYPE_TOP)}, @{$layer->regions} );
+            if (@$projection_new) {
+                # Merge the new top surfaces with the preceding top surfaces.
+                # Apply the safety offset to the newly added polygons, so they will connect
+                # with the polygons collected before,
+                # but don't apply the safety offset during the union operation as it would
+                # inflate the polygons over and over.
+                push @$buildplate_only_top_surfaces, @{ offset($projection_new, scale(0.01)) };
+                $buildplate_only_top_surfaces = union($buildplate_only_top_surfaces, 0);
+            }
+        }
+
         # detect overhangs and contact areas needed to support them
         my (@overhang, @contact) = ();
         if ($layer_id == 0) {
@@ -129,16 +166,23 @@ sub contact_area {
                 my $diff;
             
                 # If a threshold angle was specified, use a different logic for detecting overhangs.
-                if (defined $threshold_rad
-                    || $layer_id < $self->object_config->support_material_enforce_layers
-                    || ($self->object_config->raft_layers > 0 && $layer_id == 0)) {
-                    my $d = defined $threshold_rad
-                        ? scale $lower_layer->height * ((cos $threshold_rad) / (sin $threshold_rad))
-                        : 0;
+                if (($conf->support_material && defined $threshold_rad)
+                      || $layer_id <= $conf->support_material_enforce_layers
+                      || ($conf->raft_layers > 0 && $layer_id == 0)) {
+                    my $d = 0;
+                    my $layer_threshold_rad = $threshold_rad;
+                    if ($layer_id <= $conf->support_material_enforce_layers) {
+                        # Use ~45 deg number for enforced supports if we are in auto
+                        $layer_threshold_rad = deg2rad(89);
+                    }
+                    if (defined $layer_threshold_rad) {
+                        $d = scale $lower_layer->height
+                            * ((cos $layer_threshold_rad) / (sin $layer_threshold_rad));
+                    }
                 
                     $diff = diff(
-                        offset([ map $_->p, @{$layerm->slices} ], -$d),
-                        [ map @$_, @{$lower_layer->slices} ],
+                        [ map $_->p, @{$layerm->slices} ],
+                        offset([ map @$_, @{$lower_layer->slices} ], +$d),
                     );
                 
                     # only enforce spacing from the object ($fw/2) if the threshold angle
@@ -152,7 +196,7 @@ sub contact_area {
                 } else {
                     $diff = diff(
                         [ map $_->p, @{$layerm->slices} ],
-                        offset([ map @$_, @{$lower_layer->slices} ], +$fw*2),
+                        offset([ map @$_, @{$lower_layer->slices} ], +$conf->get_abs_value_over('support_material_threshold', $fw)),
                     );
                 
                     # collapse very tiny spots
@@ -164,22 +208,31 @@ sub contact_area {
                     # outside the lower slice boundary, thus no overhang
                 }
                 
-                if ($self->object_config->dont_support_bridges) {
+                if ($conf->dont_support_bridges) {
                     # compute the area of bridging perimeters
-                    # Note: this is duplicate code from GCode.pm, we need to refactor
-                    
                     my $bridged_perimeters;  # Polygons
                     {
                         my $bridge_flow = $layerm->flow(FLOW_ROLE_PERIMETER, 1);
                         
-                        my $nozzle_diameter = $self->print_config->get_at('nozzle_diameter', $layerm->region->config->perimeter_extruder-1);
-                        my $lower_grown_slices = offset([ map @$_, @{$lower_layer->slices} ], +scale($nozzle_diameter/2));
+                        # Get the lower layer's slices and grow them by half the nozzle diameter
+                        # because we will consider the upper perimeters supported even if half nozzle
+                        # falls outside the lower slices.
+                        my $lower_grown_slices;
+                        {
+                            my $nozzle_diameter = $self->print_config->get_at('nozzle_diameter', $layerm->region->config->perimeter_extruder-1);
+                            $lower_grown_slices = offset(
+                                [ map @$_, @{$lower_layer->slices} ],
+                                +scale($nozzle_diameter/2),
+                            );
+                        }
                         
-                        # TODO: split_at_first_point() could split a bridge mid-way
-                        my @overhang_perimeters =
-                            map { $_->isa('Slic3r::ExtrusionLoop') ? $_->polygon->split_at_first_point : $_->polyline->clone }
-                            map @$_, @{$layerm->perimeters};
+                        # Get all perimeters as polylines.
+                        # TODO: split_at_first_point() (called by as_polyline() for ExtrusionLoops)
+                        # could split a bridge mid-way
+                        my @overhang_perimeters = map $_->as_polyline, @{$layerm->perimeters->flatten};
                         
+                        # Only consider the overhang parts of such perimeters,
+                        # overhangs being those parts not supported by 
                         # workaround for Clipper bug, see Slic3r::Polygon::clip_as_polyline()
                         $_->[0]->translate(1,0) for @overhang_perimeters;
                         @overhang_perimeters = @{diff_pl(
@@ -201,11 +254,16 @@ sub contact_area {
                         
                         # convert bridging polylines into polygons by inflating them with their thickness
                         {
-                            # since we're dealing with bridges, we can't assume width is larger than spacing,
-                            # so we take the largest value and also apply safety offset to be ensure no gaps
-                            # are left in between
-                            my $w = max($bridge_flow->scaled_width, $bridge_flow->scaled_spacing);
+                            # For bridges we can't assume width is larger than spacing because they
+                            # are positioned according to non-bridging perimeters spacing.
+                            my $w = max(
+                                $bridge_flow->scaled_width,
+                                $bridge_flow->scaled_spacing,
+                                $fw,  # width of external perimeters
+                                $layerm->flow(FLOW_ROLE_PERIMETER)->scaled_width,
+                            );
                             $bridged_perimeters = union([
+                                # Also apply safety offset to ensure no gaps are left in between.
                                 map @{$_->grow($w/2 + 10)}, @overhang_perimeters
                             ]);
                         }
@@ -216,7 +274,7 @@ sub contact_area {
                         my @bridges = map $_->expolygon,
                             grep $_->bridge_angle != -1,
                             @{$layerm->fill_surfaces->filter_by_type(S_TYPE_BOTTOMBRIDGE)};
-                            
+                        
                         $diff = diff(
                             $diff,
                             [
@@ -235,12 +293,18 @@ sub contact_area {
                         # just remove bridged areas
                         $diff = diff(
                             $diff,
-                            [ map @$_, @{$layerm->bridged} ],
+                            $layerm->bridged,
                             1,
                         );
                     }
+                } # if ($conf->dont_support_bridges)
+
+                if ($buildplate_only) {
+                    # Don't support overhangs above the top surfaces.
+                    # This step is done before the contact surface is calcuated by growing the overhang region.
+                    $diff = diff($diff, $buildplate_only_top_surfaces);
                 }
-                
+
                 next if !@$diff;
                 push @overhang, @$diff;  # NOTE: this is not the full overhang as it misses the outermost half of the perimeter width!
             
@@ -249,11 +313,16 @@ sub contact_area {
                 # We increment the area in steps because we don't want our support to overflow
                 # on the other side of the object (if it's very thin).
                 {
-                    my @slices_margin = @{offset([ map @$_, @{$lower_layer->slices} ], +$fw/2)};
+                    my $slices_margin = offset([ map @$_, @{$lower_layer->slices} ], +$fw/2);
+                    if ($buildplate_only) {
+                        # Trim the inflated contact surfaces by the top surfaces as well.
+                        push @$slices_margin, map $_->clone, @{$buildplate_only_top_surfaces};
+                        $slices_margin = union($slices_margin);
+                    }
                     for ($fw/2, map {scale MARGIN_STEP} 1..(MARGIN / MARGIN_STEP)) {
                         $diff = diff(
                             offset($diff, $_),
-                            \@slices_margin,
+                            $slices_margin,
                         );
                     }
                 }
@@ -267,27 +336,28 @@ sub contact_area {
             # get the average nozzle diameter used on this layer
             my @nozzle_diameters = map $self->print_config->get_at('nozzle_diameter', $_),
                 map { $_->config->perimeter_extruder-1, $_->config->infill_extruder-1, $_->config->solid_infill_extruder-1 }
-                @{$layer->regions};
+                map $_->region, @{$layer->regions};
             my $nozzle_diameter = sum(@nozzle_diameters)/@nozzle_diameters;
             
             my $contact_z = $layer->print_z - $self->contact_distance($layer->height, $nozzle_diameter);
             
             # ignore this contact area if it's too low
-            next if $contact_z < $self->object_config->get_value('first_layer_height') - epsilon;
+            next if $contact_z < $conf->get_value('first_layer_height') - epsilon;
             
             $contact{$contact_z}  = [ @contact ];
             $overhang{$contact_z} = [ @overhang ];
             
             if (0) {
                 require "Slic3r/SVG.pm";
-                Slic3r::SVG::output("contact_" . $contact_z . ".svg",
-                    expolygons      => union_ex(\@contact),
-                    red_expolygons  => union_ex(\@overhang),
+                Slic3r::SVG::output("out\\contact_" . $contact_z . ".svg",
+                    green_expolygons => union_ex($buildplate_only_top_surfaces),
+                    blue_expolygons  => union_ex(\@contact),
+                    red_expolygons   => union_ex(\@overhang),
                 );
             }
         }
     }
-    
+
     return (\%contact, \%overhang);
 }
 
@@ -297,6 +367,8 @@ sub object_top {
     # find object top surfaces
     # we'll use them to clip our support and detect where does it stick
     my %top = ();  # print_z => [ expolygons ]
+    return \%top if ($self->object_config->support_material_buildplate_only);
+
     my $projection = [];
     foreach my $layer (reverse @{$object->layers}) {
         if (my @top = map @{$_->slices->filter_by_type(S_TYPE_TOP)}, @{$layer->regions}) {
@@ -429,6 +501,9 @@ sub generate_interface_layers {
 
 sub generate_bottom_interface_layers {
     my ($self, $support_z, $base, $top, $interface) = @_;
+
+    # If no interface layers are allowed, don't generate bottom interface layers.
+    return if $self->object_config->support_material_interface_layers == 0;
     
     my $area_threshold = $self->interface_flow->scaled_spacing ** 2;
     
@@ -558,11 +633,6 @@ sub generate_toolpaths {
         $pattern = 'honeycomb';
     }
     
-    my %fillers = (
-        interface   => $object->fill_maker->filler('rectilinear'),
-        support     => $object->fill_maker->filler($pattern),
-    );
-    
     my $interface_angle = $self->object_config->support_material_angle + 90;
     my $interface_spacing = $self->object_config->support_material_interface_spacing + $interface_flow->spacing;
     my $interface_density = $interface_spacing == 0 ? 1 : $interface_flow->spacing / $interface_spacing;
@@ -673,10 +743,22 @@ sub generate_toolpaths {
             $layer->support_interface_fills->append(@loops);
         }
         
+        # Allocate the fillers exclusively in the worker threads! Don't allocate them at the main thread,
+        # as Perl copies the C++ pointers by default, so then the C++ objects are shared between threads!
+        my %fillers = (
+            interface   => Slic3r::Filler->new_from_type('rectilinear'),
+            support     => Slic3r::Filler->new_from_type($pattern),
+        );
+        my $bounding_box = $object->bounding_box;
+        $fillers{interface}->set_bounding_box($object->bounding_box);
+        $fillers{support}->set_bounding_box($object->bounding_box);
+        
         # interface and contact infill
         if (@$interface || @$contact_infill) {
-            $fillers{interface}->angle($interface_angle);
-            $fillers{interface}->spacing($_interface_flow->spacing);
+            # make interface layers alternate angles by 90 degrees
+            my $alternate_angle = $interface_angle + (90 * (($layer_id + 1) % 2));
+            $fillers{interface}->set_angle(deg2rad($alternate_angle));
+            $fillers{interface}->set_min_spacing($_interface_flow->spacing);
             
             # find centerline of the external loop
             $interface = offset2($interface, +scaled_epsilon, -(scaled_epsilon + $_interface_flow->scaled_width/2));
@@ -702,7 +784,7 @@ sub generate_toolpaths {
             
             my @paths = ();
             foreach my $expolygon (@{union_ex($interface)}) {
-                my @p = $fillers{interface}->fill_surface(
+                my $p = $fillers{interface}->fill_surface(
                     Slic3r::Surface->new(expolygon => $expolygon, surface_type => S_TYPE_INTERNAL),
                     density      => $interface_density,
                     layer_height => $layer->height,
@@ -716,8 +798,8 @@ sub generate_toolpaths {
                     mm3_per_mm  => $mm3_per_mm,
                     width       => $_interface_flow->width,
                     height      => $layer->height,
-                ), @p;
-            }
+                ), @$p;
+            }                
             
             $layer->support_interface_fills->append(@paths);
         }
@@ -725,30 +807,41 @@ sub generate_toolpaths {
         # support or flange
         if (@$base) {
             my $filler = $fillers{support};
-            $filler->angle($angles[ ($layer_id) % @angles ]);
+            $filler->set_angle(deg2rad($angles[ ($layer_id) % @angles ]));
             
             # We don't use $base_flow->spacing because we need a constant spacing
             # value that guarantees that all layers are correctly aligned.
-            $filler->spacing($flow->spacing);
+            $filler->set_min_spacing($flow->spacing);
             
             my $density     = $support_density;
             my $base_flow   = $_flow;
             
             # find centerline of the external loop/extrusions
-            my $to_infill = offset2_ex($base, +scaled_epsilon, -(scaled_epsilon + $_flow->scaled_width/2));
+            my $to_infill = offset2($base, +scaled_epsilon, -(scaled_epsilon + $_flow->scaled_width/2));
             
             my @paths = ();
             
             # base flange
             if ($layer_id == 0) {
                 $filler = $fillers{interface};
-                $filler->angle($self->object_config->support_material_angle + 90);
+                $filler->set_angle(deg2rad($self->object_config->support_material_angle + 90));
                 $density        = 0.5;
                 $base_flow      = $self->first_layer_flow;
                 
                 # use the proper spacing for first layer as we don't need to align
                 # its pattern to the other layers
-                $filler->spacing($base_flow->spacing);
+                $filler->set_min_spacing($base_flow->spacing);
+                
+                # subtract brim so that it goes around the object fully (and support gets its own brim)
+                if ($self->print_config->brim_width > 0) {
+                    my $d = +scale $self->print_config->brim_width*2;
+                    $to_infill = diff_ex(
+                        $to_infill,
+                        offset($object->get_layer(0)->slices->polygons, $d),
+                    );
+                } else {
+                    $to_infill = union_ex($to_infill);
+                }
             } else {
                 # draw a perimeter all around support infill
                 # TODO: use brim ordering algorithm
@@ -759,15 +852,15 @@ sub generate_toolpaths {
                     mm3_per_mm  => $mm3_per_mm,
                     width       => $_flow->width,
                     height      => $layer->height,
-                ), map @$_, @$to_infill;
+                ), @$to_infill;
                 
                 # TODO: use offset2_ex()
-                $to_infill = offset_ex([ map @$_, @$to_infill ], -$_flow->scaled_spacing);
+                $to_infill = offset_ex($to_infill, -$_flow->scaled_spacing);
             }
             
             my $mm3_per_mm = $base_flow->mm3_per_mm;
             foreach my $expolygon (@$to_infill) {
-                my @p = $filler->fill_surface(
+                my $p = $filler->fill_surface(
                     Slic3r::Surface->new(expolygon => $expolygon, surface_type => S_TYPE_INTERNAL),
                     density     => $density,
                     layer_height => $layer->height,
@@ -780,7 +873,7 @@ sub generate_toolpaths {
                     mm3_per_mm  => $mm3_per_mm,
                     width       => $base_flow->width,
                     height      => $layer->height,
-                ), @p;
+                ), @$p;
             }
             
             $layer->support_fills->append(@paths);
