@@ -6,16 +6,19 @@ use warnings;
 BEGIN {
     use FindBin;
     use lib "$FindBin::Bin/lib";
+    use local::lib '--no-create', "$FindBin::Bin/local-lib";
 }
 
 use File::Basename qw(basename);
 use Getopt::Long qw(:config no_auto_abbrev);
 use List::Util qw(first);
-use POSIX qw(setlocale LC_NUMERIC);
+use POSIX qw(setlocale LC_NUMERIC ceil);
 use Slic3r;
+use Slic3r::Geometry qw(epsilon X Y Z deg2rad);
 use Time::HiRes qw(gettimeofday tv_interval);
 $|++;
 binmode STDOUT, ':utf8';
+binmode STDERR, ':utf8';
 
 our %opt = ();
 my %cli_options = ();
@@ -26,27 +29,33 @@ my %cli_options = ();
         
         'debug'                 => \$Slic3r::debug,
         'gui'                   => \$opt{gui},
+        'no-gui'                => \$opt{no_gui},
         'o|output=s'            => \$opt{output},
+        'j|threads=i'           => \$opt{threads},
         
         'save=s'                => \$opt{save},
         'load=s@'               => \$opt{load},
         'autosave=s'            => \$opt{autosave},
         'ignore-nonexistent-config' => \$opt{ignore_nonexistent_config},
-        'no-plater'             => \$opt{no_plater},
-        'gui-mode=s'            => \$opt{gui_mode},
         'datadir=s'             => \$opt{datadir},
         'export-svg'            => \$opt{export_svg},
         'merge|m'               => \$opt{merge},
         'repair'                => \$opt{repair},
         'cut=f'                 => \$opt{cut},
+        'cut-grid=s'            => \$opt{cut_grid},
         'split'                 => \$opt{split},
         'info'                  => \$opt{info},
         
         'scale=f'               => \$opt{scale},
-        'rotate=i'              => \$opt{rotate},
+        'rotate=f'              => \$opt{rotate},
         'duplicate=i'           => \$opt{duplicate},
         'duplicate-grid=s'      => \$opt{duplicate_grid},
         'print-center=s'        => \$opt{print_center},
+        'dont-arrange'          => \$opt{dont_arrange},
+        
+        # legacy options, ignored
+        'no-plater'             => \$opt{no_plater},
+        'gui-mode=s'            => \$opt{gui_mode},
     );
     foreach my $opt_key (keys %{$Slic3r::Config::Options}) {
         my $cli = $Slic3r::Config::Options->{$opt_key}->{cli} or next;
@@ -54,7 +63,11 @@ my %cli_options = ();
         $options{ "$opt_key|$cli" } = \$cli_options{$opt_key};
     }
     
+    @ARGV = grep !/^-psn_\d/, @ARGV if $^O eq 'darwin';
     GetOptions(%options) or usage(1);
+    
+    warn "--no-plater option is deprecated; ignoring\n" if $opt{no_plater};
+    warn "--gui-mode option is deprecated (Slic3r now has only Expert Mode); ignoring\n"  if $opt{gui_mode};
 }
 
 # load configuration files
@@ -62,61 +75,64 @@ my @external_configs = ();
 if ($opt{load}) {
     foreach my $configfile (@{$opt{load}}) {
         $configfile = Slic3r::decode_path($configfile);
-        if (-e $configfile) {
+        if (-e Slic3r::encode_path($configfile)) {
             push @external_configs, Slic3r::Config->load($configfile);
-        } elsif (-e "$FindBin::Bin/$configfile") {
+        } elsif (-e Slic3r::encode_path("$FindBin::Bin/$configfile")) {
             printf STDERR "Loading $FindBin::Bin/$configfile\n";
             push @external_configs, Slic3r::Config->load("$FindBin::Bin/$configfile");
         } else {
             $opt{ignore_nonexistent_config} or die "Cannot find specified configuration file ($configfile).\n";
         }
     }
+    
+    # expand shortcuts before applying, otherwise destination values would be already filled with defaults
+    $_->normalize for @external_configs;
 }
 
 # process command line options
-my $cli_config = Slic3r::Config->new;
-foreach my $c (@external_configs, Slic3r::Config->new_from_cli(%cli_options)) {
-    $c->normalize;  # expand shortcuts before applying, otherwise destination values would be already filled with defaults
-    $cli_config->apply($c);
-}
+my $cli_config = Slic3r::Config->new_from_cli(%cli_options);
+$cli_config->normalize;  # expand shortcuts
 
 # save configuration
 if ($opt{save}) {
-    if (@{$cli_config->get_keys} > 0) {
-        $cli_config->save($opt{save});
+    my $config = $cli_config->clone;
+    $config->apply($_) for @external_configs;
+    if (@{$config->get_keys} > 0) {
+        $config->save($opt{save});
     } else {
-        Slic3r::Config->new_from_defaults->save($opt{save});
+        Slic3r::Config->new_from_defaults->save(Slic3r::decode_path($opt{save}));
     }
 }
 
-# apply command line config on top of default config
-my $config = Slic3r::Config->new_from_defaults;
-$config->apply($cli_config);
-
 # launch GUI
 my $gui;
-if ((!@ARGV || $opt{gui}) && !$opt{save} && eval "require Slic3r::GUI; 1") {
+if ((!@ARGV || $opt{gui}) && !$opt{no_gui} && !$opt{save} && eval "require Slic3r::GUI; 1") {
     {
         no warnings 'once';
-        $Slic3r::GUI::datadir   = Slic3r::decode_path($opt{datadir} // '');
-        $Slic3r::GUI::no_plater = $opt{no_plater};
-        $Slic3r::GUI::mode      = $opt{gui_mode};
-        $Slic3r::GUI::autosave  = $opt{autosave};
+        $Slic3r::GUI::datadir       = Slic3r::decode_path($opt{datadir} // '');
+        $Slic3r::GUI::autosave      = Slic3r::decode_path($opt{autosave} // '');
+        $Slic3r::GUI::threads       = $opt{threads};
     }
     $gui = Slic3r::GUI->new;
     setlocale(LC_NUMERIC, 'C');
-    $gui->{mainframe}->load_config_file($_) for @{$opt{load}};
-    $gui->{mainframe}->load_config($cli_config);
-    foreach my $input_file (@ARGV) {
-        $input_file = Slic3r::decode_path($input_file);
-        $gui->{mainframe}{plater}->load_file($input_file) unless $opt{no_plater};
-    }
+    $gui->CallAfter(sub {
+        $gui->{mainframe}->load_config_file($_) for @{$opt{load}};
+        $gui->{mainframe}->load_config($cli_config);
+        foreach my $input_file (@ARGV) {
+            $input_file = Slic3r::decode_path($input_file);
+            $gui->{mainframe}{plater}->load_file($input_file);
+        }
+    });
     $gui->MainLoop;
     exit;
 }
-die $@ if $@ && $opt{gui};
+die $@ if $@ && $opt{gui} && !$opt{no_gui};
 
 if (@ARGV) {  # slicing from command line
+    # apply command line config on top of default config
+    my $config = Slic3r::Config->new_from_defaults;
+    $config->apply($_) for @external_configs;
+    $config->apply($cli_config);
     $config->validate;
     
     if ($opt{repair}) {
@@ -144,7 +160,7 @@ if (@ARGV) {  # slicing from command line
             $mesh->translate(0, 0, -$mesh->bounding_box->z_min);
             my $upper = Slic3r::TriangleMesh->new;
             my $lower = Slic3r::TriangleMesh->new;
-            $mesh->cut($opt{cut}, $upper, $lower);
+            $mesh->cut(Z, $opt{cut}, $upper, $lower);
             $upper->repair;
             $lower->repair;
             $upper->write_ascii("${file}_upper.stl")
@@ -155,6 +171,49 @@ if (@ARGV) {  # slicing from command line
         exit;
     }
     
+    if ($opt{cut_grid}) {
+        my ($grid_x, $grid_y) = split /[,x]/, $opt{cut_grid}, 2;
+        foreach my $file (@ARGV) {
+            $file = Slic3r::decode_path($file);
+            my $model = Slic3r::Model->read_from_file($file);
+            $model->add_default_instances;
+            my $mesh = $model->mesh;
+            my $bb = $mesh->bounding_box;
+            $mesh->translate(0, 0, -$bb->z_min);
+            
+            my $x_parts = ceil(($bb->size->x - epsilon)/$grid_x);
+            my $y_parts = ceil(($bb->size->y - epsilon)/$grid_y); #--
+            
+            for my $i (1..$x_parts) {
+                my $this = Slic3r::TriangleMesh->new;
+                if ($i == $x_parts) {
+                    $this = $mesh;
+                } else {
+                    my $next = Slic3r::TriangleMesh->new;
+                    $mesh->cut(X, $bb->x_min + ($grid_x * $i), $next, $this);
+                    $this->repair;
+                    $next->repair;
+                    $mesh = $next;
+                }
+                
+                for my $j (1..$y_parts) {
+                    my $tile = Slic3r::TriangleMesh->new;
+                    if ($j == $y_parts) {
+                        $tile = $this;
+                    } else {
+                        my $next = Slic3r::TriangleMesh->new;
+                        $this->cut(Y, $bb->y_min + ($grid_y * $j), $next, $tile);
+                        $tile->repair;
+                        $next->repair;
+                        $this = $next;
+                    }
+                    $tile->write_ascii("${file}_${i}_${j}.stl");
+                }
+            }
+        }
+        exit;
+    }
+    
     if ($opt{split}) {
         foreach my $file (@ARGV) {
             $file = Slic3r::decode_path($file);
@@ -167,7 +226,7 @@ if (@ARGV) {  # slicing from command line
             foreach my $new_mesh (@{$mesh->split}) {
                 my $output_file = sprintf '%s_%02d.stl', $file, ++$part_count;
                 printf "Writing to %s\n", basename($output_file);
-                Slic3r::Format::STL->write_file($output_file, $new_mesh, binary => 1);
+                $new_mesh->write_binary($output_file);
             }
         }
         exit;
@@ -182,6 +241,7 @@ if (@ARGV) {  # slicing from command line
         } else {
             $model = Slic3r::Model->read_from_file($input_file);
         }
+        $model->repair;
         
         if ($opt{info}) {
             $model->print_info;
@@ -197,18 +257,20 @@ if (@ARGV) {  # slicing from command line
         
         my $sprint = Slic3r::Print::Simple->new(
             scale           => $opt{scale}          // 1,
-            rotate          => $opt{rotate}         // 0,
+            rotate          => deg2rad($opt{rotate} // 0),
             duplicate       => $opt{duplicate}      // 1,
             duplicate_grid  => $opt{duplicate_grid} // [1,1],
-            print_center    => $opt{print_center}   // Slic3r::Pointf->new(100,100),
+            print_center    => $opt{print_center},
+            dont_arrange    => $opt{dont_arrange}   // 0,
             status_cb       => sub {
                 my ($percent, $message) = @_;
                 printf "=> %s\n", $message;
             },
-            output_file     => $opt{output},
+            output_file     => Slic3r::decode_path($opt{output}),
         );
         
         $sprint->apply_config($config);
+        $sprint->config->set('threads', $opt{threads}) if $opt{threads};
         $sprint->set_model($model);
         
         if ($opt{export_svg}) {
@@ -239,7 +301,7 @@ sub usage {
     my $j = '';
     if ($Slic3r::have_threads) {
         $j = <<"EOF";
-    -j, --threads <num> Number of threads to use (1+, default: $config->{threads})
+    -j, --threads <num> Number of threads to use
 EOF
     }
     
@@ -254,6 +316,9 @@ Usage: slic3r.pl [ OPTIONS ] [ file.stl ] [ file2.stl ] ...
     --save <file>       Save configuration to the specified file
     --load <file>       Load configuration from the specified file. It can be used 
                         more than once to load options from multiple files.
+    --datadir <path>    Load and store settings at the given directory.
+                        This is useful for maintaining different profiles or including
+                        configurations from a network storage.
     -o, --output <file> File to output gcode to (by default, the file will be saved
                         into the same directory as the input file using the
                         --output-filename-format to generate the filename.) If a
@@ -272,8 +337,8 @@ $j
   GUI options:
     --gui               Forces the GUI launch instead of command line slicing (if you
                         supply a model file, it will be loaded into the plater)
-    --no-plater         Disable the plater tab
-    --gui-mode          Overrides the configured mode (simple/expert)
+    --no-gui            Forces the command line slicing instead of gui. 
+                        This takes precedence over --gui if both are present.
     --autosave <file>   Automatically export current configuration to the specified file
 
   Output options:
@@ -293,7 +358,9 @@ $j
                         (default: 100,100)
     --z-offset          Additional height in mm to add to vertical coordinates
                         (+/-, default: $config->{z_offset})
-    --gcode-flavor      The type of G-code to generate (reprap/teacup/makerware/sailfish/mach3/machinekit/no-extrusion,
+    --z-steps-per-mm    Number of full steps per mm of the Z axis. Experimental feature for
+                        preventing rounding issues.
+    --gcode-flavor      The type of G-code to generate (reprap/teacup/repetier/makerware/sailfish/mach3/machinekit/smoothie/no-extrusion,
                         default: $config->{gcode_flavor})
     --use-relative-e-distances Enable this to get relative E values (default: no)
     --use-firmware-retraction  Enable firmware-controlled retraction using G10/G11 (default: no)
@@ -376,7 +443,11 @@ $j
     --fill-density      Infill density (range: 0%-100%, default: $config->{fill_density}%)
     --fill-angle        Infill angle in degrees (range: 0-90, default: $config->{fill_angle})
     --fill-pattern      Pattern to use to fill non-solid layers (default: $config->{fill_pattern})
-    --external-fill-pattern Pattern to use to fill solid layers (default: $config->{external_fill_pattern})
+    --fill-gaps         Fill gaps with single passes (default: yes)
+    --external-infill-pattern Pattern to use to fill solid layers.
+                        (Shortcut for --top-infill-pattern and --bottom-infill-pattern)
+    --top-infill-pattern Pattern to use to fill top solid layers (default: $config->{top_infill_pattern})
+    --bottom-infill-pattern Pattern to use to fill bottom solid layers (default: $config->{bottom_infill_pattern})
     --start-gcode       Load initial G-code from the supplied file. This will overwrite
                         the default command (home all axes [G28]).
     --end-gcode         Load final G-code from the supplied file. This will overwrite 
@@ -403,7 +474,7 @@ $j
     --extra-perimeters  Add more perimeters when needed (default: yes)
     --avoid-crossing-perimeters Optimize travel moves so that no perimeters are crossed (default: no)
     --thin-walls        Detect single-width walls (default: yes)
-    --overhangs         Experimental option to use bridge flow, speed and fan for overhangs
+    --detect-bridging-perimeters  Detect bridging perimeters and apply bridge flow, speed and fan
                         (default: yes)
   
    Support material options:
@@ -427,6 +498,8 @@ $j
     --support-material-enforce-layers
                         Enforce support material on the specified number of layers from bottom,
                         regardless of --support-material and threshold (0+, default: $config->{support_material_enforce_layers})
+    --support-material-buildplate-only
+                        Only create support if it lies on a build plate. Don't create support on a print. (default: no)
     --dont-support-bridges
                         Experimental option for preventing support material from being generated under bridged areas (default: yes)
   
@@ -439,6 +512,8 @@ $j
     --retract-before-travel
                         Only retract before travel moves of this length in mm (default: $config->{retract_before_travel}[0])
     --retract-lift      Lift Z by the given distance in mm when retracting (default: $config->{retract_lift}[0])
+    --retract-lift-above Only lift Z when above the specified height (default: $config->{retract_lift_above}[0])
+    --retract-lift-below Only lift Z when below the specified height (default: $config->{retract_lift_below}[0])
     --retract-layer-change
                         Enforce a retraction before each Z move (default: no)
     --wipe              Wipe the nozzle while doing a retraction (default: no)
@@ -473,6 +548,8 @@ $j
                         of filament on the first layer, for each extruder (mm, 0+, default: $config->{min_skirt_length})
     --brim-width        Width of the brim that will get added to each object to help adhesion
                         (mm, default: $config->{brim_width})
+    --interior-brim-width  Width of the brim that will get printed inside object holes to help adhesion
+                        (mm, default: $config->{interior_brim_width})
    
    Transform options:
     --scale             Factor for scaling input object (default: 1)
@@ -480,6 +557,9 @@ $j
     --duplicate         Number of items with auto-arrange (1+, default: 1)
     --duplicate-grid    Number of items with grid arrangement (default: 1,1)
     --duplicate-distance Distance in mm between copies (default: $config->{duplicate_distance})
+    --dont-arrange      Don't arrange the objects on the build plate. The model coordinates
+                        define the absolute positions on the build plate. 
+                        The option --print-center will be ignored.
     --xy-size-compensation
                         Grow/shrink objects by the configured absolute distance (mm, default: $config->{xy_size_compensation})
    
