@@ -1,4 +1,6 @@
-#include <myinit.h>
+#include <xsinit.h>
+#include "utils.hpp" 
+#include <regex>
 
 void
 confess_at(const char *file, int line, const char *func,
@@ -26,3 +28,13 @@ confess_at(const char *file, int line, const char *func,
      LEAVE;
     #endif
 }
+
+std::vector<std::string> 
+split_at_regex(const std::string& input, const std::string& regex) {
+    // passing -1 as the submatch index parameter performs splitting
+    std::regex re(regex);
+    std::sregex_token_iterator
+        first{input.begin(), input.end(), re, -1},
+        last;
+    return {first, last};
+}
